#include "GaddBox.h"

// GADDBOX INITIALISE MEMBER VARIABLES
GaddBox::GaddBox() :
	m_Scroll(false),
	m_Select(false),
	m_Move(false),
	m_Add(false),
	m_FlockP(false),
	m_SquareP(false),
	m_CircleP(false),
	m_LineP(false),
	m_Formation(false),
	m_Control(false),
	m_Selecting(false),
	m_ButtonPressed(false),
	m_OPressed(false),
	BL(false),
	BR(false),
	TL(false),
	TR(false),
	m_Counter(0),
	m_Scale(0.0f),
	m_Padding(0),
	m_IsPlayer(false),
	m_IsAgent(false),
	m_Debug(false),
	m_SerialID(0),
	m_Speed(0.0f),
	m_AngularSpeed(0.0f),
	m_Mines(0),
	m_IsFire(false),
	m_BulletSpeed(0.0f),
	m_Update(false),
	m_Timer(0.0f),
	m_Up(false),
	m_Down(false),
	m_Left(false),
	m_Right(false),
	m_Release(false),
	m_MaxPinValue(0.0f),
	m_Acceleration(0.0f),
	m_Increment(0.0f)
{}

// DE-CONSTRUCTOR
GaddBox::~GaddBox() {


}
 
// INIT GADDBOX
bool GaddBox::init(cocos2d::Scene* scene, bool debug) {

	m_Scene = scene; // SCENE OBJECT

	// WINDOW SIZES
	m_WindowSize = cocos2d::Director::getInstance()->getWinSize();
	m_VisibleSize = cocos2d::Director::getInstance()->getVisibleSize();

	// SET SESSION DATA
	m_SessionData->getInstance()->setWinWidth(m_WindowSize.width);
	m_SessionData->getInstance()->setWinHeight(m_WindowSize.height);

	// GET MOBILE BOOLEAN AND SCALE
	m_IsMobile = m_SessionData->getInstance()->getMobile();
	m_Scale = m_SessionData->getInstance()->getScale();

	// BOX2D INIT
	b2Vec2 gravity;
	gravity.Set(0.0f, 0.0f);
	m_B2World = new b2World(gravity);
	m_B2World->SetAllowSleeping(m_DoSleep);
	m_B2World->SetContinuousPhysics(true);
	m_B2World->SetContactListener(&myContactListenerInstance);
	m_SessionData->getInstance()->setB2World(m_B2World);

	// MARK CORNER POINTS
	HUD_TR = cocos2d::Vec2(m_VisibleSize.width / 2, m_VisibleSize.height / 2);
	HUD_TL = cocos2d::Vec2(-(m_VisibleSize.width / 2), m_VisibleSize.height / 2);
	HUD_BR = cocos2d::Vec2(m_VisibleSize.width / 2, -(m_VisibleSize.height / 2));
	HUD_BL = cocos2d::Vec2(-(m_VisibleSize.width / 2), -(m_VisibleSize.height / 2));

	// BASE LAYER IS OBLY USED FOR THE GRID SO FAR
	m_BaseLayer = cocos2d::Layer::create();
	m_BaseLayer->setAnchorPoint(cocos2d::Vec2(0.5, 0.5));
	m_BaseLayer->setPosition(cocos2d::Vec2(m_WindowSize.width / 2, m_WindowSize.height / 2));
	m_Scene->addChild(m_BaseLayer, 0);

	// MAIN LAYER WHERE ALL OBJECTS ARE LOCATED
	m_MainLayer = cocos2d::Layer::create();
	m_MainLayer->setAnchorPoint(cocos2d::Vec2(0.5, 0.5));
	m_MainLayer->setPosition(cocos2d::Vec2(m_WindowSize.width / 2, m_WindowSize.height / 2));
	m_Scene->addChild(m_MainLayer, 1);
	m_SessionData->getInstance()->setMainLayer(m_MainLayer);

	// HUD LAYER INIT
	m_HUDLayer = cocos2d::Layer::create();
	m_HUDLayer->setContentSize(m_VisibleSize);
	m_HUDLayer->setIgnoreAnchorPointForPosition(true);
	m_HUDLayer->setAnchorPoint(cocos2d::Vec2(0.5, 0.5));
	m_HUDLayer->setPosition(cocos2d::Vec2(m_WindowSize.width / 2, m_WindowSize.height / 2));
	m_Scene->addChild(m_HUDLayer, 2);

	if (debug) {

		m_Debug = true;

		m_DebugDraw = B2DebugDrawLayer::create(m_B2World);
		m_MainLayer->addChild(m_DebugDraw, 12);

	}

	// THIS NODE LAYER IS USED FOR DRAWING PRIMITIVE OBJECTS
	m_DrawNode = cocos2d::DrawNode::create();
	m_MainLayer->addChild(m_DrawNode, 3);
	m_SessionData->getInstance()->setDrawNode(m_DrawNode);

	// HIGHLIGHT RECTANGE 
	m_Highlight = cocos2d::DrawNode::create();
	m_HUDLayer->addChild(m_Highlight, 3);

	// CACHE 
	m_Cache = cocos2d::SpriteFrameCache::getInstance();

	m_Cursor = cocos2d::DrawNode::create();
	m_HUDLayer->addChild(m_Cursor, 3);
	m_Cursor->setOpacity(30);

	m_SelectRect = cocos2d::DrawNode::create();
	m_SelectRect->setPosition(cocos2d::Vec2(0, 0));
	m_HUDLayer->addChild(m_SelectRect, 2);

	srand(time(NULL));

	m_SerialID = 0;

	m_Update = true;
	m_Forming = false;
	m_FlockP = true;
	m_FormationPressed = false;
	m_Gap = 100.0f;

	m_Form = 1;

	m_Utils->getInstance()->initCount();

	m_Marker = new Marker();

	return true;

}

// UPDATE GADDBOX
void GaddBox::update(float delta) {

	if (m_Update) {
	
		// STEP BOX2D PHYSCIS CALCULATIONS
		int32 velocityIterations = 3;
		int32 positionIterations = 3;
		m_B2World->Step(delta, velocityIterations, positionIterations);

		m_DrawNode->clear();
		m_Highlight->clear();

		if (m_Selecting) { m_Selected.clear(); } // SOMETIMES WE WANT TO RETAIN WHAT OBJECTA ARE SELECTED 

		// UPDATE ALL OBJECTS
		for (m_GameObjects_Iter = m_GameObjects.begin(); m_GameObjects_Iter != m_GameObjects.end();) {

			(*m_GameObjects_Iter).second->update();

			if (m_Selecting) { // IF SELECTING IS TRUE WE NEED TO CHECK WHAT IS INSIDE THE RECTANGE AREA

				if (m_Area.containsPoint((*m_GameObjects_Iter).second->getPosition())) {

					(*m_GameObjects_Iter).second->setSelected(true);

					m_Selected.insert((*m_GameObjects_Iter));

					m_DrawNode->drawCircle((*m_GameObjects_Iter).second->getPosition(), (*m_GameObjects_Iter).second->getSize().width / 2, (*m_GameObjects_Iter).second->getRotation(), 12, false, cocos2d::Color4F(1.0f, 0.0f, 0.0f, 1.0f));

				}
				
				else {

					(*m_GameObjects_Iter).second->setSelected(false);

				}
			
			}

			// ELSE DRAW HIGHLIGHT CIRLCE IF ALREADY SELECTED
			else if ((*m_GameObjects_Iter).second->getSelected()) {

				float angle = (*m_GameObjects_Iter).second->getRotation();

				m_DrawNode->drawCircle((*m_GameObjects_Iter).second->getPosition(), (*m_GameObjects_Iter).second->getSize().width / 2, (*m_GameObjects_Iter).second->getRotation(), 12, false, cocos2d::Color4F(1.0f, 0.0f, 0.0f, 1.0f));

			}

			++m_GameObjects_Iter;

		}

		// CHECK WHICH BUTTONS NEED TO BE HIGHLIGHTED
		for (auto button : m_GFormation) {

			if (m_FlockP) {
			
				m_Highlight->drawRect(cocos2d::Vec2(m_BFlock->getPosition().x - m_BFlock->getBoundingBox().size.width / 2, m_BFlock->getPosition().y - m_BFlock->getBoundingBox().size.height / 2), cocos2d::Vec2(m_BFlock->getPosition().x + m_BFlock->getBoundingBox().size.width / 2, m_BFlock->getPosition().y + m_BFlock->getBoundingBox().size.height / 2), cocos2d::Color4F(1.0f, 1.0f, 1.0f, 1.0f));
			
			}

			else if (m_SquareP) {

				m_Highlight->drawRect(cocos2d::Vec2(m_BSquare->getPosition().x - m_BSquare->getBoundingBox().size.width / 2, m_BSquare->getPosition().y - m_BSquare->getBoundingBox().size.height / 2), cocos2d::Vec2(m_BSquare->getPosition().x + m_BSquare->getBoundingBox().size.width / 2, m_BSquare->getPosition().y + m_BSquare->getBoundingBox().size.height / 2), cocos2d::Color4F(1.0f, 1.0f, 1.0f, 1.0f));

			}

			else if (m_CircleP) {

				m_Highlight->drawRect(cocos2d::Vec2(m_BCircle->getPosition().x - m_BCircle->getBoundingBox().size.width / 2, m_BCircle->getPosition().y - m_BCircle->getBoundingBox().size.height / 2), cocos2d::Vec2(m_BCircle->getPosition().x + m_BCircle->getBoundingBox().size.width / 2, m_BCircle->getPosition().y + m_BCircle->getBoundingBox().size.height / 2), cocos2d::Color4F(1.0f, 1.0f, 1.0f, 1.0f));

			}

			else if (m_LineP) {

				m_Highlight->drawRect(cocos2d::Vec2(m_BLine->getPosition().x - m_BLine->getBoundingBox().size.width / 2, m_BLine->getPosition().y - m_BLine->getBoundingBox().size.height / 2), cocos2d::Vec2(m_BLine->getPosition().x + m_BLine->getBoundingBox().size.width / 2, m_BLine->getPosition().y + m_BLine->getBoundingBox().size.height / 2), cocos2d::Color4F(1.0f, 1.0f, 1.0f, 1.0f));

			}
		
			if (m_Scroll) {

				m_Highlight->drawRect(cocos2d::Vec2(m_BScroll->getPosition().x - m_BScroll->getBoundingBox().size.width / 2, m_BScroll->getPosition().y - m_BScroll->getBoundingBox().size.height / 2), cocos2d::Vec2(m_BScroll->getPosition().x + m_BScroll->getBoundingBox().size.width / 2, m_BScroll->getPosition().y + m_BScroll->getBoundingBox().size.height / 2), cocos2d::Color4F(1.0f, 1.0f, 1.0f, 1.0f));

			}

			else if (m_Select) {

				m_Highlight->drawRect(cocos2d::Vec2(m_BSelect->getPosition().x - m_BSelect->getBoundingBox().size.width / 2, m_BSelect->getPosition().y - m_BSelect->getBoundingBox().size.height / 2), cocos2d::Vec2(m_BSelect->getPosition().x + m_BSelect->getBoundingBox().size.width / 2, m_BSelect->getPosition().y + m_BSelect->getBoundingBox().size.height / 2), cocos2d::Color4F(1.0f, 1.0f, 1.0f, 1.0f));

			}

			else if (m_Move) {

				m_Highlight->drawRect(cocos2d::Vec2(m_BMove->getPosition().x - m_BMove->getBoundingBox().size.width / 2, m_BMove->getPosition().y - m_BMove->getBoundingBox().size.height / 2), cocos2d::Vec2(m_BMove->getPosition().x + m_BMove->getBoundingBox().size.width / 2, m_BMove->getPosition().y + m_BMove->getBoundingBox().size.height / 2), cocos2d::Color4F(1.0f, 1.0f, 1.0f, 1.0f));

			}

			else if (m_Add) {

				m_Highlight->drawRect(cocos2d::Vec2(m_BAdd->getPosition().x - m_BAdd->getBoundingBox().size.width / 2, m_BAdd->getPosition().y - m_BAdd->getBoundingBox().size.height / 2), cocos2d::Vec2(m_BAdd->getPosition().x + m_BAdd->getBoundingBox().size.width / 2, m_BAdd->getPosition().y + m_BAdd->getBoundingBox().size.height / 2), cocos2d::Color4F(1.0f, 1.0f, 1.0f, 1.0f));

			}

		}

		m_Marker->setRotation(-atan2(m_OnTouchMovePoint.y - m_TransTouchPoint.y, m_OnTouchMovePoint.x - m_TransTouchPoint.x) * RTD_RATIO);
		m_Marker->update();

	}

	else {

		// STEP BOX2D PHYSCIS CALCULATIONS
		int32 velocityIterations = 1;
		int32 positionIterations = 1;
		m_B2World->Step(delta, velocityIterations, positionIterations);

		// UPDATE ALL OBJECTS
		for (m_GameObjects_Iter = m_GameObjects.begin(); m_GameObjects_Iter != m_GameObjects.end();) {

			(*m_GameObjects_Iter).second->update();
			++m_GameObjects_Iter;

		}

	}

}

// ON TOUCH BEGIN 
cocos2d::Vec2 GaddBox::onTouchBegan(cocos2d::Touch* touch, cocos2d::Event* event) {

	// VARIOUS LAYER AND TOUCH CALCULATIONS ARE REQUIRED TO MANAGE THE VARIOUS LAYERS AND CONTROL THEM
	// WE ALSO FORCE POINT ZERO ZERO TO BE CENTER OF DISPLAY
	m_OnTouchPoint = touch->getLocation();
	normalizeTouch = cocos2d::Vec2(m_OnTouchPoint.x - ((m_WindowSize.width - m_VisibleSize.width) / 2), m_OnTouchPoint.y - ((m_WindowSize.height - m_VisibleSize.height) / 2));
	m_TransTouchPoint = cocos2d::Vec2(normalizeTouch.x - (m_VisibleSize.width / 2), normalizeTouch.y - (m_VisibleSize.height / 2));
	m_OnMainLayerTouchPoint = m_TransTouchPoint - (m_MainLayer->getPosition() - m_HUDLayer->getPosition());
	m_OnTouchMovePoint = cocos2d::Vec2(0, 0);
	m_LayerOrigin = m_BaseLayer->getPosition();

	// WE CHECK IF BUTTON IS PRESSED
	if (checkButtonPressed(m_TransTouchPoint)) { // IF BUTTON NOT PRESSED

		if (m_Scroll) {



		}
		
		else if (m_Select) {

			if (!m_FormationPressed) {
			
				for (auto object : m_Selected) { object.second->setSelected(false); }
				m_Selected.clear();
				m_FormationPressed = false;
			
			}
			

		}
		
		else if (m_Move) {



		}
		
		else if (m_Add) {

			for (auto object : m_Selected) { object.second->setSelected(false); }
			m_Selected.clear();

		}

	}
	
	// ELSE WE CHECK WHICH BUTTONS ARE ACTIVE AND ACT ACCORDINGLY 
	else {

		if (m_Scroll) {

			// NO FUNCTIONALITY HERE YET

		}

		else if (m_Select) {

			for (auto object : m_Selected) { object.second->setSelected(false); }
			m_Selected.clear();

		}
		
		// THIS IS A CRITICAL PART OF THE API & NEEDS TO BE CAREFULLY STUDIED 
		else if (m_Move) {

			// IF NOTHING IS SELECTED THEN WE DO NOT NEED TO ENTER
			if (m_Selected.size() > 0) {

				// WHICH FORMATION IS CURRENTLY SELECTED
				switch (m_Form) {

					// tHESE SWITCHES CURRENLTY ARE NOT REQUIRED - I THOUGHT WE MAY NEED SOME DIFFERENCES BETWEEN DEVELOPMENT
					// FOR NOW THEY ARE THE SAME. wE ARE USING THE MARKER FOR CASE 2, BUT IT ISNT REQUIRED AT MOMENT

					case 1: {

						setMoveController(); // THIS FUCNTION ORGANISES AND MANAGES THE SELECTED VECTOR OF OBJECTS. iT RELIEVES ALL OBJECTS OF THEIR RESPONSIBILITIES 


					}break;

					case 2: {

						// IN THIS EXAMPLE WE ARE USING A PLACEHOLDER THAT REPRESENTS THE AGENTS POTENTIOIAL POSITION
						// THIS CARRIED ONTO TOUCH END WHERE WE USE THIS BOOLEAN 

						m_Forming = true;

						m_Marker->init(m_Selected.size());
						m_Marker->setPosition(m_OnMainLayerTouchPoint);


					}break;

					case 3: {

						setMoveController(); // THIS FUCNTION ORGANISES AND MANAGES THE SELECTED VECTOR OF OBJECTS. iT RELIEVES ALL OBJECTS OF THEIR RESPONSIBILITIES 

					}break;

					case 4: {

						setMoveController(); // THIS FUCNTION ORGANISES AND MANAGES THE SELECTED VECTOR OF OBJECTS. iT RELIEVES ALL OBJECTS OF THEIR RESPONSIBILITIES 

					}break;

					default : {



					}break;
				
				}
			
			}

		}

		else if (m_Add) {

			for (auto object : m_Selected) { object.second->setSelected(false); }
			m_Selected.clear();

			m_Utils->getInstance()->increment();
			GameObject* object = new GameObject();
			object->createSprite("circleAgent.png");
			object->setScale(m_Scale);
			object->setPosition(m_OnMainLayerTouchPoint);
			object->init(2, 1.0f, 1.0f, 1.0f, false);
			m_MainLayer->addChild(object, 1);
			m_GameObjects.insert(std::pair<int, GameObject*>(object->getID(), object));

		}
	
	}

	return m_OnMainLayerTouchPoint;

}

// ON TOUCH MOVE 
void GaddBox::onTouchMoved(cocos2d::Touch* touch, cocos2d::Event* event) {

	normalizeMoveTouch = cocos2d::Vec2(touch->getLocation().x - ((m_WindowSize.width - m_VisibleSize.width) / 2), touch->getLocation().y - ((m_WindowSize.height - m_VisibleSize.height) / 2));
	m_OnTouchMovePoint = cocos2d::Vec2(normalizeMoveTouch.x - (m_VisibleSize.width / 2), normalizeMoveTouch.y - (m_VisibleSize.height / 2));

	// IF A CONTROL BUTTON IS ACTIVE WE NEED TOI CHECK HER WHICH ONES ARE ACTIVE AND ACT UPON THEM
	if (!m_ButtonPressed) {
		 
		// WE USE THIS TO RE-SET THE MAIN LAYER POSITION BY USING THE ORIGONAL TOUCH POINT AND THE MOVE TOUCH POINT FROM THE TOUCH FUNCTIONS
		if (m_Scroll) {

			m_BaseLayer->setPosition(m_LayerOrigin + ((m_OnTouchMovePoint - m_TransTouchPoint) / 2));
			m_MainLayer->setPosition(m_LayerOrigin + ((m_OnTouchMovePoint - m_TransTouchPoint) / 2));
			m_MainLayerPosition = cocos2d::Vec2((-(m_MainLayer->getPosition().x - (m_VisibleSize.width / 2))), (-(m_MainLayer->getPosition().y - (m_VisibleSize.height / 2))));

		}

		// THIS SETS THE SIZE AND POSITION OF THE SELECT RECTANGE - IT DOESNT SET OBJECTS AS SELECTEED, WE DO THAT IN THE UPDATE FUNCTION
		else if (m_Select) {

			m_Selecting = true;

			m_SelectRect->clear();
			m_SelectRect->drawRect(m_TransTouchPoint, cocos2d::Vec2(m_TransTouchPoint.x, m_OnTouchMovePoint.y), m_OnTouchMovePoint, cocos2d::Vec2(m_OnTouchMovePoint.x, m_TransTouchPoint.y), cocos2d::Color4F(1, 1, 0, 1));

			m_Area = cocos2d::Rect(
				m_TransTouchPoint.x < m_OnTouchMovePoint.x ? (m_TransTouchPoint.x - (m_MainLayer->getPosition().x - m_HUDLayer->getPosition().x)) : (m_OnTouchMovePoint.x - (m_MainLayer->getPosition().x - m_HUDLayer->getPosition().x)),
				m_TransTouchPoint.y < m_OnTouchMovePoint.y ? (m_TransTouchPoint.y - (m_MainLayer->getPosition().y - m_HUDLayer->getPosition().y)) : (m_OnTouchMovePoint.y - (m_MainLayer->getPosition().y - m_HUDLayer->getPosition().y)),
				m_TransTouchPoint.x < m_OnTouchMovePoint.x ? (m_OnTouchMovePoint.x - (m_TransTouchPoint.x)) : (m_TransTouchPoint.x - (m_OnTouchMovePoint.x)),
				m_TransTouchPoint.y < m_OnTouchMovePoint.y ? (m_OnTouchMovePoint.y - (m_TransTouchPoint.y)) : (m_TransTouchPoint.y - (m_OnTouchMovePoint.y))
			);

		}

		else if (m_Move) {



		}

		else if (m_Add) {



		}

	}

}

// ON TOUCH END 
void GaddBox::onTouchEnded(cocos2d::Touch* touch, cocos2d::Event* event) {

	if (m_Forming) {

		setMoveController();

		m_Marker->clear();

		m_Forming = false;

	}

	m_SelectRect->clear();
	m_Selecting = false;
	m_ButtonPressed = false;

}

// ON TOUCH CANCEL 
void GaddBox::onTouchCancelled(cocos2d::Touch* touch, cocos2d::Event* event) {

	if (m_Forming) {

		setMoveController();

		m_Marker->clear();

		m_Forming = false;

	}

	m_SelectRect->clear();
	m_Selecting = false;
	m_ButtonPressed = false;
}

// CACHED SPRITE SHEET
void GaddBox::cacheSpriteSheet(std::string file) {

	m_Cache->addSpriteFramesWithFile(file);

}

// HALT AN OBJECTS RESPONSIBILITIES - THIS ALLOWS THE DEVELOPER TO DO OTHER THINGS WITH THE OBJECT
void GaddBox::objectHalt(int serialID) {

	// WE FIND THE OBJECT USING FIND AND SERIAL ID
	GameObject* object = m_GameObjects.find(serialID)->second;

	// WE RELIEVE THE OBJECT OF ANY LEADERSHIP RESPONSIBILITIES
	if (object->getLeading()) {

		object->setFollow(false);
		object->setLeader(nullptr);
		object->setLeading(false);

		std::map<int, GameObject*> objects;
		std::map<int, GameObject*>::iterator objects_Iter;

		for (objects_Iter = object->getGameObjects().begin(); objects_Iter != object->getGameObjects().end();) {

			(*objects_Iter).second->setFollow(false);
			(*objects_Iter).second->setLeader(nullptr);
			(*objects_Iter).second->setLeading(false);

			if ((*objects_Iter).second->getSelected() == false) {

				objects.insert((*objects_Iter));
				objects_Iter = object->getGameObjects().erase(objects_Iter);

			}

			else {

				++objects_Iter;

			}

		}

		object->setLeading(false);

		object->getGameObjects().clear();

		if (objects.size() > 0) { setLeader(objects, object->getTarget(), object->getFormation(), object->getRestAngle()); }

	}

	// WE HALT ANY FOLLOWING BEHAVIOURS
	else if (object->getFollow()) {

		object->stopFollow();

	}

}

// DELETE AN OBJECT - FIRST HALT OBJECT
void GaddBox::objectDelete(int serialID) {

	// WE FIND THE OBJECT USING ITS SERIAL ID
	GameObject* object = m_GameObjects.find(serialID)->second;

	// WE RELIEVE THE OBJECT OF ANY LEADERSHIP RESPONSIBILITIES
	if (object->getLeading()) {

		object->setFollow(false);
		object->setLeader(nullptr);
		object->setLeading(false);

		std::map<int, GameObject*> objects;
		std::map<int, GameObject*>::iterator objects_Iter;

		for (objects_Iter = object->getGameObjects().begin(); objects_Iter != object->getGameObjects().end();) {

			(*objects_Iter).second->setFollow(false);
			(*objects_Iter).second->setLeader(nullptr);
			(*objects_Iter).second->setLeading(false);

			if ((*objects_Iter).second->getSelected() == false) {

				objects.insert((*objects_Iter));
				objects_Iter = object->getGameObjects().erase(objects_Iter);

			}

			else {

				++objects_Iter;

			}

		}

		object->setLeading(false);

		object->getGameObjects().clear();

		if (objects.size() > 0) { setLeader(objects, object->getTarget(), object->getFormation(), object->getRestAngle()); }

	}

	// WE HALT ANY FOLLOWING BEHAVIOURS
	else if (object->getFollow()) {

		object->stopFollow();

	}

	// DELETING OBJECT
	m_GameObjects.erase(serialID);
	object->clean();
	m_MainLayer->removeChild(object);

}

// INIT BACKGROUND GRID
void GaddBox::initGrid() {

	int x = 0;
	int y = 0;

	auto drawNode = cocos2d::DrawNode::create();
	m_BaseLayer->addChild(drawNode);
	cocos2d::Size m_Size = m_BaseLayer->getBoundingBox().size;

	for (x = -2000; x <= 2000; x += m_Gap) {

		drawNode->drawLine(cocos2d::Vec2(x, -2000), cocos2d::Vec2(x, 2000), cocos2d::Color4F(1.0f, 0.0f, 0.0f, 0.1f));
		drawNode->drawLine(cocos2d::Vec2(x, -2000), cocos2d::Vec2(x, 2000), cocos2d::Color4F(1.0f, 0.0f, 0.0f, 0.1f));

	}

	x = 0;
	y = 0;

	for (y = -2000; y <= 2000; y += m_Gap) {

		drawNode->drawLine(cocos2d::Vec2(-2000, y), cocos2d::Vec2(2000, y), cocos2d::Color4F(1.0f, 0.0f, 0.0f, 0.1f));
		drawNode->drawLine(cocos2d::Vec2(-2000, y), cocos2d::Vec2(2000, y), cocos2d::Color4F(1.0f, 0.0f, 0.0f, 0.1f));

	}

}

void GaddBox::activateHUDBorder() {

	auto drawNode = cocos2d::DrawNode::create();
	drawNode->setAnchorPoint(cocos2d::Vec2(0.5, 0.5));
	drawNode->setPosition(cocos2d::Vec2(0, 0));

	drawNode->drawLine(HUD_BL, HUD_BR, cocos2d::Color4F(1.0f, 1.0f, 0.0f, 0.1f));
	drawNode->drawLine(HUD_BR, HUD_TR, cocos2d::Color4F(1.0f, 1.0f, 0.0f, 0.1f));
	drawNode->drawLine(HUD_TR, HUD_TL, cocos2d::Color4F(1.0f, 1.0f, 0.0f, 0.1f));
	drawNode->drawLine(HUD_TL, HUD_BL, cocos2d::Color4F(1.0f, 1.0f, 0.0f, 0.1f));

	m_HUDLayer->addChild(drawNode);

}

// USE TWO AREA BOXES TO CHECK IF ANY OF OUR CONTROL OR FORMATION BUTTONS HAVE BEEN PRESSED
bool GaddBox::checkButtonPressed(cocos2d::Vec2 location) {

	// WE NEED TO CHECK THIS RECTANGE AREA. IF THE TOUCH POINT IS INSIDE THIS RECTANLGE , THEN WE CAN CHECK EVERY BUTTON INDIVIDUALLY
	if (m_ControlRect.containsPoint(m_TransTouchPoint)) {

		for (auto control : m_GControl) {

			auto rect = control->getBoundingBox();

			if (rect.containsPoint(location)) { //RECT CHECK START

				m_ButtonPressed = true;

				if (control->getTag() == 1) {
					m_Scroll = true;
					m_Select = false;
					m_Move = false;
					m_Add = false;
					CCLOG("SCROLL PRESSED");
				}

				else if (control->getTag() == 2) {
					m_Scroll = false;
					m_Select = true;
					m_Move = false;
					m_Add = false;
					CCLOG("SELECT PRESSED");
				}

				else if (control->getTag() == 3) {
					m_Scroll = false;
					m_Select = false;
					m_Move = true;
					m_Add = false;
					CCLOG("MOVE PRESSED");
				}

				else if (control->getTag() == 4) {
					m_Scroll = false;
					m_Select = false;
					m_Move = false;
					m_Add = true;
					CCLOG("ADD PRESSED");
				}

				return true;

			} // RECT CHECK END

		} // LOOP

	} // OUTER RECT CHECK

	  // WE NEED TO CHECK THIS RECTANGE AREA. IF THE TOUCH POINT IS INSIDE THIS RECTANLGE , THEN WE CAN CHECK EVERY BUTTON INDIVIDUALLY
	if (m_FormationRect.containsPoint(m_TransTouchPoint)) {

		for (auto formation : m_GFormation) {

			auto m_RectOne = formation->getBoundingBox();

			if (m_RectOne.containsPoint(location)) { //RECT CHECK START

				m_ButtonPressed = true;
				m_FormationPressed = true;

				if (formation->getTag() == 1) {
					m_FlockP = true;
					m_SquareP = false;
					m_CircleP = false;
					m_LineP = false;
					m_Form = 1;
					CCLOG("FLOCK PRESSED");}

				else if (formation->getTag() == 2) {
					m_FlockP = false;
					m_SquareP = true;
					m_CircleP = false;
					m_LineP = false;
					m_Form = 2;
					CCLOG("SQUARE PRESSED");}

				else if (formation->getTag() == 3) {
					m_FlockP = false;
					m_SquareP = false;
					m_CircleP = true;
					m_LineP = false;
					m_Form = 3;
					CCLOG("CIRCLE PRESSED");}

				else if (formation->getTag() == 4) {
					m_FlockP = false;
					m_SquareP = false;
					m_CircleP = false;
					m_LineP = true;
					m_Form = 4;
					CCLOG("LINE PRESSED");}



				return true;

			} // RECT CHECK END

		} // LOOP

	} // OUTER RECT CHECK

	return false;

}

// FUNCTION TO CHECK IF A POPINT IS INSIDE A CIRLE RADIUS AREA
bool GaddBox::pointInsideCircle(cocos2d::Vec2 point, cocos2d::Vec2 center, float radius) {

	float distance = sqrt(pow(center.x - point.x, 2) + pow(center.y - point.y, 2));

	return distance <= radius;

}

// SET CONTROL UI CONTROL PANEL - TESTING VERSION
bool GaddBox::setUIControlTesting(std::string location, std::string scroll, std::string select, std::string move, std::string add, float used, std::string panel) {

	if (!m_Control) {

		m_ControlPanel = cocos2d::Sprite::createWithSpriteFrameName(panel);
		m_ControlPanel->setScale(m_Scale);
		cocos2d::Size size = m_ControlPanel->getBoundingBox().size;
		size.width = size.width * used;
		m_HUDLayer->addChild(m_ControlPanel, 0);
		float segment = size.width / 13;

		m_BScroll = cocos2d::Sprite::createWithSpriteFrameName(scroll);
		m_BScroll->setScale(m_Scale);
		m_BScroll->setTag(1);
		m_HUDLayer->addChild(m_BScroll);
		m_GControl.push_back(m_BScroll);
		m_Scroll = true;

		m_BSelect = cocos2d::Sprite::createWithSpriteFrameName(select);
		m_BSelect->setScale(m_Scale);
		m_BSelect->setTag(2);
		m_HUDLayer->addChild(m_BSelect);
		m_GControl.push_back(m_BSelect);

		m_BMove = cocos2d::Sprite::createWithSpriteFrameName(move);
		m_BMove->setScale(m_Scale);
		m_BMove->setTag(3);
		m_HUDLayer->addChild(m_BMove);
		m_GControl.push_back(m_BMove);

		m_BAdd = cocos2d::Sprite::createWithSpriteFrameName(add);
		m_BAdd->setScale(m_Scale);
		m_BAdd->setTag(4);
		m_HUDLayer->addChild(m_BAdd);
		m_GControl.push_back(m_BAdd);

		if (location == "BL") { // PANEL BOTTOM LEFT

			if (!BL) { // BOTTOM LEFT AVAILABLE ?

				m_ControlPanel->setAnchorPoint(cocos2d::Vec2(0, 0));
				m_ControlPanel->setPosition(HUD_BL);

				m_BScroll->setPosition(cocos2d::Vec2(HUD_BL.x + (segment * 2), HUD_BL.y + (size.height / 2)));

				m_BSelect->setPosition(cocos2d::Vec2(HUD_BL.x + (segment * 5), HUD_BL.y + (size.height / 2)));

				m_BMove->setPosition(cocos2d::Vec2(HUD_BL.x + (segment * 8), HUD_BL.y + (size.height / 2)));

				m_BAdd->setPosition(cocos2d::Vec2(HUD_BL.x + (segment * 11), HUD_BL.y + (size.height / 2)));

				BL = true; // BOTTOM LEFT IS NOW USED

				m_Control = true;

				m_ControlRect = cocos2d::Rect(cocos2d::Vec2((m_BScroll->getPosition().x) - m_BScroll->getBoundingBox().size.width / 2, (HUD_BL.y + (size.height / 2)) - m_BScroll->getBoundingBox().size.height / 2), cocos2d::Size(11.0f * segment, m_BScroll->getBoundingBox().size.height));

				return true;

			}
			else { return false; }

		}
		
		else if (location == "BR") { // PANEL BOTTOM LEFT

			if (!BR) { // BOTTOM RIGHT AVAILABLE ?

				m_ControlPanel->setAnchorPoint(cocos2d::Vec2(1, 0));
				m_ControlPanel->setPosition(HUD_BR);
				m_ControlPanel->setFlippedX(true);

				m_BScroll->setPosition(cocos2d::Vec2(HUD_BR.x - (segment * 2), HUD_BR.y + (size.height / 2)));

				m_BSelect->setPosition(cocos2d::Vec2(HUD_BR.x - (segment * 5), HUD_BR.y + (size.height / 2)));

				m_BMove->setPosition(cocos2d::Vec2(HUD_BR.x - (segment * 8), HUD_BR.y + (size.height / 2)));

				m_BAdd->setPosition(cocos2d::Vec2(HUD_BR.x - (segment * 11), HUD_BR.y + (size.height / 2)));

				BR = true;

				m_Control = true;

				m_ControlRect = cocos2d::Rect(cocos2d::Vec2((m_BScroll->getPosition().x) - m_BScroll->getBoundingBox().size.width / 2, (HUD_BL.y + (size.height / 2)) - m_BScroll->getBoundingBox().size.height / 2), cocos2d::Size(11.0f * segment, m_BScroll->getBoundingBox().size.height));

				return true;

			}
			else { return false; }

		}
		
		else if (location == "TL") { // PANEL BOTTOM LEFT

			if (!TL) { // BOTTOM RIGHT AVAILABLE ?

				m_ControlPanel->setAnchorPoint(cocos2d::Vec2(0, 1));
				m_ControlPanel->setPosition(HUD_TL);
				m_ControlPanel->setFlippedY(true);

				m_BScroll->setPosition(cocos2d::Vec2(HUD_TL.x + (segment * 2), HUD_TL.y - (size.height / 2)));

				m_BSelect->setPosition(cocos2d::Vec2(HUD_TL.x + (segment * 5), HUD_TL.y - (size.height / 2)));

				m_BMove->setPosition(cocos2d::Vec2(HUD_TL.x + (segment * 8), HUD_TL.y - (size.height / 2)));

				m_BAdd->setPosition(cocos2d::Vec2(HUD_TL.x + (segment * 11), HUD_TL.y - (size.height / 2)));

				TL = true;

				m_Control = true;

				m_ControlRect = cocos2d::Rect(cocos2d::Vec2((m_BScroll->getPosition().x) - m_BScroll->getBoundingBox().size.width / 2, (HUD_BL.y + (size.height / 2)) - m_BScroll->getBoundingBox().size.height / 2), cocos2d::Size(11.0f * segment, m_BScroll->getBoundingBox().size.height));

				return true;

			}
			else { return false; }

		}
		
		else if (location == "TR") { // PANEL BOTTOM LEFT

			if (!TR) { // BOTTOM RIGHT AVAILABLE ?

				m_ControlPanel->setAnchorPoint(cocos2d::Vec2(1, 1));
				m_ControlPanel->setPosition(HUD_TR);
				m_ControlPanel->setFlippedX(true);
				m_ControlPanel->setFlippedY(true);

				m_BScroll->setPosition(cocos2d::Vec2(HUD_TR.x - (segment * 2), HUD_TR.y - (size.height / 2)));

				m_BSelect->setPosition(cocos2d::Vec2(HUD_TR.x - (segment * 5), HUD_TR.y - (size.height / 2)));

				m_BMove->setPosition(cocos2d::Vec2(HUD_TR.x - (segment * 8), HUD_TR.y - (size.height / 2)));

				m_BAdd->setPosition(cocos2d::Vec2(HUD_TR.x - (segment * 11), HUD_TR.y - (size.height / 2)));

				TR = true;

				m_Control = true;

				m_ControlRect = cocos2d::Rect(cocos2d::Vec2((m_BScroll->getPosition().x) - m_BScroll->getBoundingBox().size.width / 2, (HUD_BL.y + (size.height / 2)) - m_BScroll->getBoundingBox().size.height / 2), cocos2d::Size(11.0f * segment, m_BScroll->getBoundingBox().size.height));

				return true;

			}
			else { return false; }

		}

	}
	
	else { return false; }

	return false;

}

// SET CONTROL UI CONTROL PANEL
bool GaddBox::setUIControl(std::string location, std::string scroll, std::string select, std::string move, float used, std::string panel) {

	if (!m_Control) {

		m_ControlPanel = cocos2d::Sprite::createWithSpriteFrameName(panel);
		m_ControlPanel->setScale(m_Scale);
		cocos2d::Size size = m_ControlPanel->getBoundingBox().size;
		size.width = size.width * used;
		m_HUDLayer->addChild(m_ControlPanel, 0);
		float segment = size.width / 13;

		m_BScroll = cocos2d::Sprite::createWithSpriteFrameName(scroll);
		m_BScroll->setScale(m_Scale);
		m_BScroll->setTag(1);
		m_HUDLayer->addChild(m_BScroll);
		m_GControl.push_back(m_BScroll);
		m_Scroll = true;

		m_BSelect = cocos2d::Sprite::createWithSpriteFrameName(select);
		m_BSelect->setScale(m_Scale);
		m_BSelect->setTag(2);
		m_HUDLayer->addChild(m_BSelect);
		m_GControl.push_back(m_BSelect);

		m_BMove = cocos2d::Sprite::createWithSpriteFrameName(move);
		m_BMove->setScale(m_Scale);
		m_BMove->setTag(3);
		m_HUDLayer->addChild(m_BMove);
		m_GControl.push_back(m_BMove);

		if (location == "BL") { // PANEL BOTTOM LEFT

			if (!BL) { // BOTTOM LEFT AVAILABLE ?

				m_ControlPanel->setAnchorPoint(cocos2d::Vec2(0, 0));
				m_ControlPanel->setPosition(cocos2d::Vec2(HUD_BL.x - (segment * 3), HUD_BL.y));

				m_BScroll->setPosition(cocos2d::Vec2(HUD_BL.x + (segment * 2), HUD_BL.y + (size.height / 2)));

				m_BSelect->setPosition(cocos2d::Vec2(HUD_BL.x + (segment * 5), HUD_BL.y + (size.height / 2)));

				m_BMove->setPosition(cocos2d::Vec2(HUD_BL.x + (segment * 8), HUD_BL.y + (size.height / 2)));

				BL = true; // BOTTOM LEFT IS NOW USED

				m_Control = true;

				m_ControlRect = cocos2d::Rect(cocos2d::Vec2((m_BScroll->getPosition().x) - m_BScroll->getBoundingBox().size.width / 2, (HUD_BL.y + (size.height / 2)) - m_BScroll->getBoundingBox().size.height / 2), cocos2d::Size(11.0f * segment, m_BScroll->getBoundingBox().size.height));

				return true;

			}
			else { return false; }

		}

		else if (location == "BR") { // PANEL BOTTOM LEFT

			if (!BR) { // BOTTOM RIGHT AVAILABLE ?

				m_ControlPanel->setAnchorPoint(cocos2d::Vec2(1, 0));
				m_ControlPanel->setPosition(HUD_BR);
				m_ControlPanel->setFlippedX(true);

				m_BScroll->setPosition(cocos2d::Vec2(HUD_BR.x - (segment * 2), HUD_BR.y + (size.height / 2)));

				m_BSelect->setPosition(cocos2d::Vec2(HUD_BR.x - (segment * 5), HUD_BR.y + (size.height / 2)));

				m_BMove->setPosition(cocos2d::Vec2(HUD_BR.x - (segment * 8), HUD_BR.y + (size.height / 2)));

				m_BAdd->setPosition(cocos2d::Vec2(HUD_BR.x - (segment * 11), HUD_BR.y + (size.height / 2)));

				BR = true;

				m_Control = true;

				m_ControlRect = cocos2d::Rect(cocos2d::Vec2((m_BScroll->getPosition().x) - m_BScroll->getBoundingBox().size.width / 2, (HUD_BL.y + (size.height / 2)) - m_BScroll->getBoundingBox().size.height / 2), cocos2d::Size(11.0f * segment, m_BScroll->getBoundingBox().size.height));

				return true;

			}
			else { return false; }

		}

		else if (location == "TL") { // PANEL BOTTOM LEFT

			if (!TL) { // BOTTOM RIGHT AVAILABLE ?

				m_ControlPanel->setAnchorPoint(cocos2d::Vec2(0, 1));
				m_ControlPanel->setPosition(HUD_TL);
				m_ControlPanel->setFlippedY(true);

				m_BScroll->setPosition(cocos2d::Vec2(HUD_TL.x + (segment * 2), HUD_TL.y - (size.height / 2)));

				m_BSelect->setPosition(cocos2d::Vec2(HUD_TL.x + (segment * 5), HUD_TL.y - (size.height / 2)));

				m_BMove->setPosition(cocos2d::Vec2(HUD_TL.x + (segment * 8), HUD_TL.y - (size.height / 2)));

				m_BAdd->setPosition(cocos2d::Vec2(HUD_TL.x + (segment * 11), HUD_TL.y - (size.height / 2)));

				TL = true;

				m_Control = true;

				m_ControlRect = cocos2d::Rect(cocos2d::Vec2((m_BScroll->getPosition().x) - m_BScroll->getBoundingBox().size.width / 2, (HUD_BL.y + (size.height / 2)) - m_BScroll->getBoundingBox().size.height / 2), cocos2d::Size(11.0f * segment, m_BScroll->getBoundingBox().size.height));

				return true;

			}
			else { return false; }

		}

		else if (location == "TR") { // PANEL BOTTOM LEFT

			if (!TR) { // BOTTOM RIGHT AVAILABLE ?

				m_ControlPanel->setAnchorPoint(cocos2d::Vec2(1, 1));
				m_ControlPanel->setPosition(HUD_TR);
				m_ControlPanel->setFlippedX(true);
				m_ControlPanel->setFlippedY(true);

				m_BScroll->setPosition(cocos2d::Vec2(HUD_TR.x - (segment * 2), HUD_TR.y - (size.height / 2)));

				m_BSelect->setPosition(cocos2d::Vec2(HUD_TR.x - (segment * 5), HUD_TR.y - (size.height / 2)));

				m_BMove->setPosition(cocos2d::Vec2(HUD_TR.x - (segment * 8), HUD_TR.y - (size.height / 2)));

				m_BAdd->setPosition(cocos2d::Vec2(HUD_TR.x - (segment * 11), HUD_TR.y - (size.height / 2)));

				TR = true;

				m_Control = true;

				m_ControlRect = cocos2d::Rect(cocos2d::Vec2((m_BScroll->getPosition().x) - m_BScroll->getBoundingBox().size.width / 2, (HUD_BL.y + (size.height / 2)) - m_BScroll->getBoundingBox().size.height / 2), cocos2d::Size(11.0f * segment, m_BScroll->getBoundingBox().size.height));

				return true;

			}
			else { return false; }

		}

	}

	else { return false; }

	return false;

}

// SET FORMATION UI CONTROL PANEL
bool GaddBox::setUIFormation(std::string location, std::string flock, std::string square, std::string circle, std::string line, float used, std::string panel) {

	if (!m_Formation) {

		m_FormationPanel = cocos2d::Sprite::createWithSpriteFrameName(panel);
		m_FormationPanel->setScale(m_Scale);
		cocos2d::Size size = m_FormationPanel->getBoundingBox().size;
		size.width = size.width * used;
		m_HUDLayer->addChild(m_FormationPanel, 0);
		float segment = size.width / 13;

		m_BFlock = cocos2d::Sprite::createWithSpriteFrameName("flock.png");
		m_BFlock->setScale(m_Scale);
		m_BFlock->setTag(1);
		m_HUDLayer->addChild(m_BFlock);
		m_GFormation.push_back(m_BFlock);

		m_BSquare = cocos2d::Sprite::createWithSpriteFrameName("square.png");
		m_BSquare->setScale(m_Scale);
		m_BSquare->setTag(2);
		m_HUDLayer->addChild(m_BSquare);
		m_GFormation.push_back(m_BSquare);

		m_BCircle = cocos2d::Sprite::createWithSpriteFrameName("circle.png");
		m_BCircle->setScale(m_Scale);
		m_BCircle->setTag(3);
		m_HUDLayer->addChild(m_BCircle);
		m_GFormation.push_back(m_BCircle);

		m_BLine = cocos2d::Sprite::createWithSpriteFrameName("line.png");
		m_BLine->setScale(m_Scale);
		m_BLine->setTag(4);
		m_HUDLayer->addChild(m_BLine);
		m_GFormation.push_back(m_BLine);

		if (location == "BL") { // PANEL BOTTOM LEFT

			if (!BL) { // BOTTOM LEFT AVAILABLE ?

				m_FormationPanel->setAnchorPoint(cocos2d::Vec2(0, 0));
				m_FormationPanel->setPosition(HUD_BL);

				m_BFlock->setPosition(cocos2d::Vec2(HUD_BL.x + (segment * 2), HUD_BL.y + (size.height / 2)));

				m_BSquare->setPosition(cocos2d::Vec2(HUD_BL.x + (segment * 5), HUD_BL.y + (size.height / 2)));

				m_BCircle->setPosition(cocos2d::Vec2(HUD_BL.x + (segment * 8), HUD_BL.y + (size.height / 2)));

				m_BLine->setPosition(cocos2d::Vec2(HUD_BL.x + (segment * 11), HUD_BL.y + (size.height / 2)));

				BL = true; // BOTTOM LEFT IS NOW USED

				m_Formation = true;

				m_FormationRect = cocos2d::Rect(cocos2d::Vec2((m_BFlock->getPosition().x) - m_BFlock->getBoundingBox().size.width / 2, (HUD_BL.y + (size.height / 2)) - m_BFlock->getBoundingBox().size.height / 2), cocos2d::Size(11.0f * segment, m_BFlock->getBoundingBox().size.height));

				return true;

			}
			else { return false; }

		}
		
		else if (location == "BR") { // PANEL BOTTOM LEFT

			if (!BR) { // BOTTOM RIGHT AVAILABLE ?

				m_FormationPanel->setAnchorPoint(cocos2d::Vec2(1, 0));
				m_FormationPanel->setPosition(HUD_BR);
				m_FormationPanel->setFlippedX(true);

				m_BFlock->setPosition(cocos2d::Vec2(HUD_BR.x - (segment * 2), HUD_BR.y + (size.height / 2)));

				m_BSquare->setPosition(cocos2d::Vec2(HUD_BR.x - (segment * 5), HUD_BR.y + (size.height / 2)));

				m_BCircle->setPosition(cocos2d::Vec2(HUD_BR.x - (segment * 8), HUD_BR.y + (size.height / 2)));

				m_BLine->setPosition(cocos2d::Vec2(HUD_BR.x - (segment * 11), HUD_BR.y + (size.height / 2)));

				BR = true;

				m_Formation = true;

				m_FormationRect = cocos2d::Rect(cocos2d::Vec2((m_BFlock->getPosition().x) - m_BFlock->getBoundingBox().size.width / 2, (HUD_BL.y + (size.height / 2)) - m_BFlock->getBoundingBox().size.height / 2), cocos2d::Size(11.0f * segment, m_BFlock->getBoundingBox().size.height));

				return true;

			}
			else { return false; }

		}
		
		else if (location == "TL") { // PANEL BOTTOM LEFT

			if (!TL) { // BOTTOM RIGHT AVAILABLE ?

				m_FormationPanel->setAnchorPoint(cocos2d::Vec2(0, 1));
				m_FormationPanel->setPosition(HUD_TL);
				m_FormationPanel->setFlippedY(true);

				m_BFlock->setPosition(cocos2d::Vec2(HUD_TL.x + (segment * 2), HUD_TL.y - (size.height / 2)));

				m_BSquare->setPosition(cocos2d::Vec2(HUD_TL.x + (segment * 5), HUD_TL.y - (size.height / 2)));

				m_BCircle->setPosition(cocos2d::Vec2(HUD_TL.x + (segment * 8), HUD_TL.y - (size.height / 2)));

				m_BLine->setPosition(cocos2d::Vec2(HUD_TL.x + (segment * 11), HUD_TL.y - (size.height / 2)));

				TL = true;

				m_Formation = true;

				m_FormationRect = cocos2d::Rect(cocos2d::Vec2((m_BFlock->getPosition().x) - m_BFlock->getBoundingBox().size.width / 2, (HUD_BL.y + (size.height / 2)) - m_BFlock->getBoundingBox().size.height / 2), cocos2d::Size(11.0f * segment, m_BFlock->getBoundingBox().size.height));

				return true;

			}
			else { return false; }

		}
		
		else if (location == "TR") { // PANEL BOTTOM LEFT

			if (!TR) { // BOTTOM RIGHT AVAILABLE ?

				m_FormationPanel->setAnchorPoint(cocos2d::Vec2(1, 1));
				m_FormationPanel->setPosition(HUD_TR);
				m_FormationPanel->setFlippedX(true);
				m_FormationPanel->setFlippedY(true);

				m_BLine->setPosition(cocos2d::Vec2(HUD_TR.x - (segment * 2), HUD_TR.y - (size.height / 2)));

				m_BCircle->setPosition(cocos2d::Vec2(HUD_TR.x - (segment * 5), HUD_TR.y - (size.height / 2)));

				m_BSquare->setPosition(cocos2d::Vec2(HUD_TR.x - (segment * 8), HUD_TR.y - (size.height / 2)));

				m_BFlock->setPosition(cocos2d::Vec2(HUD_TR.x - (segment * 11), HUD_TR.y - (size.height / 2)));

				TR = true;

				m_Formation = true;

				m_FormationRect = cocos2d::Rect(cocos2d::Vec2((m_BFlock->getPosition().x) - m_BFlock->getBoundingBox().size.width / 2, (HUD_TR.y - (size.height / 2)) - m_BFlock->getBoundingBox().size.height / 2), cocos2d::Size(11.0f * segment, m_BFlock->getBoundingBox().size.height));
				
				return true;

			}
			else { return false; }

		}

	}
	else { return false; }

	return false;

}

// ORGANISE AND MANAGE OBJECT RESPONSIBILITY IN TYHE SELECTED VECTOR BEFORE TELLING THE SELECTED TO FORM GROUP
void GaddBox::setMoveController() {

	for (auto selected : m_Selected) {

		if (selected.second->getLeading()) {

			std::map<int, GameObject*> objects;
			std::map<int, GameObject*>::iterator objects_Iter;

			for (objects_Iter = selected.second->getGameObjects().begin(); objects_Iter != selected.second->getGameObjects().end();) {

				(*objects_Iter).second->setFollow(false);
				(*objects_Iter).second->setLeader(nullptr);
				(*objects_Iter).second->setLeading(false);

				if ((*objects_Iter).second->getSelected() == false) {

					objects.insert((*objects_Iter));
					objects_Iter = selected.second->getGameObjects().erase(objects_Iter);

				}

				else {

					++objects_Iter;

				}

			}

			selected.second->setLeading(false);

			selected.second->getGameObjects().clear();

			if (objects.size() > 0) { setLeader(objects, selected.second->getTarget(), selected.second->getFormation(), selected.second->getRestAngle()); }

		}

		else if (selected.second->getFollow()) {

			selected.second->stopFollow();

		}

	}

	float angle = atan2(m_OnTouchMovePoint.y - m_TransTouchPoint.y, m_OnTouchMovePoint.x - m_TransTouchPoint.x) * RTD_RATIO;

	setLeader(m_Selected, m_OnMainLayerTouchPoint, m_Form, angle);

}

// SET A GROUP INTO FORMATION
void GaddBox::setLeader(std::map<int, GameObject*> objects, cocos2d::Vec2 target, int formation, float restAngle) {

	if (!objects.empty()) { // RUN TIME ERROR HANDLING
	
		// SET FIRST OBJECT AS LEADER AND CHECK IF ANYONE ELSE IN CLOSER CENTER OF OBJECTS TO ASSUME LEADER
		GameObject* leader = objects.begin()->second;
		leader->setFollow(false);
		leader->setLeading(true);

		cocos2d::Vec2 centerPoint = getCenterPoint(objects);

		m_GameObjects_Iter = objects.begin(); m_GameObjects_Iter++; // ITERATOR STARTS AT POSITION TWO - FIRST POSITION IS ASSUMED LEADER FOR NOW UNTIL PROVEN OTHERWISE

		for (m_GameObjects_Iter; m_GameObjects_Iter != objects.end();) {

			(*m_GameObjects_Iter).second->setFollow(true);
			(*m_GameObjects_Iter).second->setLeading(false);
			(*m_GameObjects_Iter).second->setLeader(nullptr);

			if (proximity((*m_GameObjects_Iter).second->getPosition(), leader->getPosition(), centerPoint)) {

				// CHECK IF THIS OBJECT IS CLOSER CENTER OF FLOCK TO ASSUME LEADER ROLE
				leader->setFollow(true);
				leader->setLeading(false);
				leader = (*m_GameObjects_Iter).second;
				leader->setFollow(false);
				leader->setLeading(true);

			}

			++m_GameObjects_Iter;

		}

		std::map<int, GameObject*> resultObjects;

		// ASSIGN LEADER TO FLOCK
		for (auto object : objects) {

			if (object.second->getFollow()) {

				object.second->setLeader(leader);
				object.second->setFormation(formation);
				object.second->setRestAngle(restAngle);
				resultObjects.insert(object);

			}

		}

		leader->setGameObjects(resultObjects);
		leader->setLeading(target, formation, restAngle);
	
	}
	else {

		CCLOG("SET LEADER CALLED WITH EMPTY OBJECTS VECTOR");

	}


}

// CHECK IF A POINT IS WITHIN PROXIMITY
bool GaddBox::proximity(cocos2d::Vec2 client, cocos2d::Vec2 leader, cocos2d::Vec2 center) {

	bool x = false;
	bool y = false;

	float leaderR = sqrt(pow(center.x - leader.x , 2) + pow(center.y - leader.y, 2));
	float clientR = sqrt(pow(center.x - client.x, 2) + pow(center.y - client.y, 2));

	if (clientR < leaderR) { return true; }

	return false;

}

// ADD AN GAME OBJECT TO THE MAIN LAYER
int GaddBox::addObject(std::string fileNameFromCache, cocos2d::Vec2 position, float density, float friction, float restitution, int type) {

	m_Utils->getInstance()->increment();
	GameObject* object = new GameObject();
	object->createSprite(fileNameFromCache);
	object->setScale(m_Scale);
	object->setPosition(position);
	object->init(type, density, friction, restitution, false);
	m_MainLayer->addChild(object, 1);
	m_GameObjects.insert(std::pair<int, GameObject*>(object->getID(), object));

	return m_Utils->getInstance()->getCount();
}

// RETRIEVE GAME OBJECT 
GameObject* GaddBox::getGameObjectBySerial(int serial) {

	return m_GameObjects.find(serial)->second;

}

// GET AVERAGE CENTER POINT OF A VECTOR OF GAME OBJECTS
// WE USE THIS TO DECIDE UPON A LEADER
// WE USE IT FOR FLOCKING FORMATION
// WE USE IT MANY OTHER PLACES TOO
cocos2d::Vec2 GaddBox::getCenterPoint(std::map<int, GameObject*>& objects) {

	float x = 0.0f;
	float y = 0.0f;

	for (auto object : objects) {

		x += object.second->getPosition().x;
		y += object.second->getPosition().y;

	}

	x = x / objects.size();
	y = y / objects.size();

	return cocos2d::Vec2(x, y);

}

