#ifndef GADDBOX_H
#define GADDBOX_H

#include "cocos2d.h"
#include "Box2D\Box2D.h"

#include "ContactListener.h"
#include "B2DebugDrawLayer.h"
#include "GLESRender.h"

#include "SessionData.h"
#include "Utils.h"
#include "AudioHandler.h"

#include "GameObject.h"

#include "Marker.h"

class GaddBox {

	public:

		GaddBox();
		~GaddBox();

		// VOID FUNCTIONS
		void update(float delta); // UPDATE FUCNTION

		cocos2d::Vec2 onTouchBegan(cocos2d::Touch* touch, cocos2d::Event* event); // TOUCH START EVENT
		void onTouchEnded(cocos2d::Touch* touch, cocos2d::Event* event);  // TOUCH MOVE EVENT
		void onTouchMoved(cocos2d::Touch* touch, cocos2d::Event* event); // TOUCH END EVENT
		void onTouchCancelled(cocos2d::Touch* touch, cocos2d::Event* event); // TOUCH END EVENT

		void initGrid(); // INITIALISE THE BACKGROUND GRID
		void activateHUDBorder(); 
		void setLeader(std::map<int, GameObject*> objects, cocos2d::Vec2 target, int form, float rest); // SET A LEADER INTO ACTION WITH A GROUP OF FOLLOWERS. THIS STARTS A FORMATION - THIS CAN BE USED MANUALLY WITHOUT USING THE FORMATION BUTTONS
		void setMoveController(); // SORTS THROUGH SELECTED OBJECTS AND ANAGES ALL THEIR RESPONSIBILITIES
		void cacheSpriteSheet(std::string file); // CACHE A SPRITE LIST / SHEET
		void objectHalt(int serialID); // TELL AN OBJCT TO HALT ALL API RESPONSIBILITIES
		void objectDelete(int serialID); // DELETE AN OBJECT
		void setSpace(float space) { m_SessionData->getInstance()->setSpace(space); }; // SET SPACE VALUE - USED IN THE FORMATION SPACING LOGIC
		void setDistance(float distance) { m_SessionData->getInstance()->setDistance(distance); }; // SET DISTANCE VALUE - USED IN THE FORMATION SPACING LOGIC

		// FUNCTIONS THAT RETURN A BOOLEAN
		bool init(cocos2d::Scene* scene, bool debug); // INITIALISE THE GADDBOX OBJECT
		bool checkButtonPressed(cocos2d::Vec2 location); // WE CHECK IF A BUTTON HAS BEEN PRESSED EVERYTIME A TOUCH HAS BEEN ACTIVATED. THERE IS SOME OPTIMISATION BENIFITS TO THIS
		bool pointInsideCircle(cocos2d::Vec2 point, cocos2d::Vec2 center, float radius); // CHECK IF A POINT IS INSIDE A CIRCLE AREA
		bool setUIControlTesting(std::string location, std::string scroll, std::string select, std::string move, std::string add, float used, std::string panel); // INIT TEST CONTROLS BUTTONS
		bool setUIControl(std::string location, std::string scroll, std::string select, std::string move, float used, std::string panel); // INIT CONTROLS BUTTONS
		bool setUIFormation(std::string location, std::string flock, std::string square, std::string circle, std::string line, float used, std::string panel); // INIT FORMATION BUTTONS
		bool proximity(cocos2d::Vec2 client, cocos2d::Vec2 leader, cocos2d::Vec2 center); // CHECK IF AN OBJECT IS WITHIN A PROXIMITY TO ANOTHER POINT

		// WE CAN CREATE AN OBJECT AND RETRUN ITS SERIAL ID
		int addObject(std::string fileNameFromCache, cocos2d::Vec2 position, float density, float friction, float restitution, int type);

		// WE GET AVERAGE CENTER POINT FROM A VECTOR OF OBJECTS
		cocos2d::Vec2 getCenterPoint(std::map<int, GameObject*>& objects);

		// RETRIEVE A GAME OBJECT USING ITS SERIAL ID
		GameObject* getGameObjectBySerial(int serial);

		// GET BOX2D WORLD OBJECT - THE DEVELOPER WILL NEED THIS TO MAKE THEIR OWN GAME LOGIC AND MECHANICS
		b2World* getB2World() { return m_B2World; };

		cocos2d::DrawNode* getDrawNode() { return m_DrawNode; };

		std::map<int, GameObject*>& getGameObjects() { return m_GameObjects; };

	private:

		cocos2d::Size m_WindowSize;
		cocos2d::Size m_VisibleSize;
		cocos2d::Scene* m_Scene;

		// BOX2D OBJECTS
		b2World* m_B2World;
		B2DebugDrawLayer* m_DebugDraw;
		ContactListener myContactListenerInstance;

		SessionData* m_SessionData; // HOLDS SESSION DATA 
		Utils* m_Utils; // MAKES UTILITY FUCNTIONS AVAILABLE 
		AudioHandler* m_Audio; // AUDIO ENGINE MADE AVAILABLE 

		// COCOS LAYERS
		cocos2d::Layer* m_BaseLayer;
		cocos2d::Layer* m_MainLayer;
		cocos2d::Layer* m_HUDLayer;

		// CORNER POSITION FOR ALL DISPLAY SIZES - VERY USEFUL HAVE A HANDLE ON
		cocos2d::Vec2 HUD_TR;
		cocos2d::Vec2 HUD_TL;
		cocos2d::Vec2 HUD_BR;
		cocos2d::Vec2 HUD_BL;

		// LAYER CONTROL
		cocos2d::Vec2 m_LayerDiff;
		cocos2d::Vec2 normalizeTouch;
		cocos2d::Vec2 normalizeMoveTouch;

		bool m_Scroll;
		bool m_Select;
		bool m_Move;
		bool m_Add;

		bool m_Forming;

		bool m_FlockP;
		bool m_SquareP;
		bool m_CircleP;
		bool m_LineP;

		int m_Form;

		bool m_Formation;
		bool m_Control;
		bool m_Selecting;
		bool m_ButtonPressed;
		bool m_OPressed;
		bool m_FormationPressed;

		bool BL;
		bool BR;
		bool TL;
		bool TR;

		bool m_Up;
		bool m_Down;
		bool m_Left;
		bool m_Right;
		bool m_Release;

		bool m_IsPlayer;
		bool m_IsAgent;

		
		float m_Scale;
		float m_Padding;
		float m_Speed;
		float m_Angle;
		float m_AngularSpeed;
		float m_MaxPinValue;
		float m_BulletSpeed;
		float m_Acceleration;
		float m_Timer;
		float m_Increment;
		float m_AgentSize;

		bool m_DoSleep;
		bool m_Debug;
		bool m_IsTipped;
		bool m_IsFire;
		bool m_Dead;
		bool m_Update;
		bool m_IsMobile;

		int m_SerialID;
		int m_Mines;
		int m_Asteroids;
		int m_Obstacles;
		int m_Artifacts;
		int m_Counter;
		int m_Gap;

		cocos2d::Vec2 m_OnTouchPoint;
		cocos2d::Vec2 m_OnTouchMovePoint;
		cocos2d::Vec2 m_TransTouchPoint;
		cocos2d::Vec2 m_OnTransTouchMovePoint;
		cocos2d::Vec2 m_MainLayerPosition;
		cocos2d::Vec2 m_OnMainLayerTouchPoint;
		cocos2d::Vec2 m_LayerOrigin;

		cocos2d::Rect m_FormationRect;
		cocos2d::Rect m_ControlRect;
		cocos2d::Rect m_Area;

		cocos2d::DrawNode* m_Cursor;
		cocos2d::DrawNode* m_SelectRect;

		cocos2d::DrawNode* m_DrawNode;
		cocos2d::DrawNode* m_Highlight;
		cocos2d::DrawNode* m_DirectionLine;
		cocos2d::DrawNode* m_WayPoint;
		cocos2d::Size m_Size;

		// TEXTURE CAHCE
		cocos2d::SpriteFrameCache* m_Cache;

		// PLACEHOLDER MARKER  - ITS IS DISPLAYED WHEN WE TOUCH SCREEN WHEN USING MOVE COMMAND
		Marker* m_Marker;

		// MAP OF OBJECTS IN GAME
		std::map<int, GameObject*> m_Selected;
		std::map<int, GameObject*> m_GameObjects;
		std::map<int, GameObject*>::iterator m_GameObjects_Iter;

		// VECTOR OF BUTTONS
		std::vector<cocos2d::Sprite*> m_GControl;
		std::vector<cocos2d::Sprite*> m_GFormation;

		// PANEL SPRITES
		cocos2d::Sprite* m_ControlPanel;
		cocos2d::Sprite* m_FormationPanel;

		// BUTTON SPRITES
		cocos2d::Sprite* m_BScroll;
		cocos2d::Sprite* m_BSelect;
		cocos2d::Sprite* m_BMove;
		cocos2d::Sprite* m_BAdd;
		cocos2d::Sprite* m_BFlock;
		cocos2d::Sprite* m_BSquare;
		cocos2d::Sprite* m_BCircle;
		cocos2d::Sprite* m_BLine;

};
#endif // DEFINE - GADDBOX_H