#include "GaddBoxScene.h"

cocos2d::Scene* GaddBoxScene::createScene() {

	return GaddBoxScene::create();

}

// SCENE INIT FUNCTION
bool GaddBoxScene::init() {

	m_WindowSize = cocos2d::Director::getInstance()->getWinSize();
	m_VisibleSize = cocos2d::Director::getInstance()->getVisibleSize();

	initTouch(); // INIT TOUCH LISTENERS

	m_Gaddbox = new GaddBox(); // CONSTRUCT GADDBOX OBJECT
	m_Gaddbox->init(this, false); // INIT GADDBOX - WE INSERT THE SCENE AND IF WE WANT TO DISPLAY BOX2D DEBUG DATA OR NOT

	m_Gaddbox->initGrid(); // INIT A BACKGROUND GRID TO HELP DEBUG YOUR GAME

	m_Gaddbox->cacheSpriteSheet("sprite_sheet.plist"); // WE NEED TO CACHE A PLIST OF SPRITES
	 
	// SET UI CONTROLS  - WE NEED TO PICK A CORNER - TOP LEFT = TL   TOP RIGHT = TR   BOTTOM EFT = BL   BOTTOM RIGHT = BR
	// WE NEED TO SELECTED A CACHED SPRITE FOR OUR PANEL AND BUTTONS. 
	m_Gaddbox->setUIControl("BL", "scroll.png", "select.png", "move.png", 0.58f, "panel.png");

	// FORMATION LOGIC IS SAME AS CONTROL LOGIC
	m_Gaddbox->setUIFormation("TR", "flock.png", "square.png", "circle.png", "line.png", 0.58f, "panel.png");

	this->scheduleUpdate();

	m_Base = cocos2d::Sprite::create("building/building.png");
	m_Base->setPosition(cocos2d::Vec2(0.0f, 0.0f));
	m_SessionData->getInstance()->getMainLayer()->addChild(m_Base, 0);

	m_Spawn = cocos2d::Vec2(m_Base->getBoundingBox().size.width * 0.25f, -m_Base->getBoundingBox().size.width * 0.25f);

	m_Players.push_back(m_Gaddbox->addObject("circleAgent.png", m_Spawn, 1.0f, 0.2f, 0.2f, 2));

	for (int x = 400; x <= 480; x+=20) {
	
		for (int y = 80; y >= -80; y-=20) {

			m_Utils->getInstance()->increment();
			GameObject* enemy = new GameObject();
			enemy->createSprite("circleAgent.png");
			enemy->setScale(m_SessionData->getInstance()->getScale());
			enemy->setPosition(cocos2d::Vec2(float(x), float(y)));
			enemy->init(2, 0.5f, 0.2f, 0.2f, false);
			m_SessionData->getInstance()->getMainLayer()->addChild(enemy, 1);
			m_Enemies.insert(std::pair<int, GameObject*>(enemy->getID(), enemy));

		}
	
	}

	m_DrawNode = m_Gaddbox->getDrawNode();

	m_GaddBoxObjects = m_Gaddbox->getGameObjects();

	return true;
}

void GaddBoxScene::update(float delta) {

	// GADDBOX MUST BE UPDATED EVERY UPDATE CALL
	m_Gaddbox->update(delta);

	for (auto player : m_Players) {
	
		for (auto enemy : m_Enemies) {

			GameObject* playerObject = m_Gaddbox->getGameObjectBySerial(player);

			if (m_Utils->getInstance()->inCircle(enemy.second->getPosition(), playerObject->getPosition(), 100.0f)) {

				playerObject->setTarget(playerObject->getPosition());
				playerObject->setHasTarget(false);
			
				if (!playerObject->getHalted()) { m_Gaddbox->objectHalt(player); }

				float angle = atan2(enemy.second->getPosition().y - playerObject->getPosition().y, enemy.second->getPosition().x - playerObject->getPosition().x) * RTD_RATIO;
				playerObject->setAngle(angle);

				if (!playerObject->m_ButtonActive) {
				
					playerObject->m_ButtonActive = true;
					playerObject->m_Bullet->x = playerObject->getPosition().x;
					playerObject->m_Bullet->y = playerObject->getPosition().y;
					playerObject->m_Bullet->myAngle = angle;
				
				}
				break;
			}
			
			else if (m_Utils->getInstance()->inCircle(enemy.second->getPosition(), playerObject->getPosition(), 150.0f) && !playerObject->getHasTarget()) {

				if (!playerObject->getHalted()) { m_Gaddbox->objectHalt(player); }

				playerObject->setTarget(enemy.second->getPosition());
				playerObject->setHasTarget(true);

			}

		}
	

	}

	m_GaddBoxObjects = m_Gaddbox->getGameObjects();

	for (auto object : m_GaddBoxObjects) {

		if (object.second->m_ButtonActive) {
		
			for (m_EnemiesIter = m_Enemies.begin(); m_EnemiesIter != m_Enemies.end();) {

				cocos2d::Rect rect = (*m_EnemiesIter).second->getBoundingBox();

				if (rect.containsPoint(cocos2d::Vec2(object.second->m_Bullet->x, object.second->m_Bullet->y))) {

					
					(*m_EnemiesIter).second->clean();
					m_SessionData->getInstance()->getMainLayer()->removeChild((*m_EnemiesIter).second);
					m_EnemiesIter = m_Enemies.erase(m_EnemiesIter);
					object.second->m_ButtonActive = false;

				}
				else {

					m_EnemiesIter++;

				}

			}

			if (object.second->m_ButtonActive) {

				float posX = 2 * cos(object.second->m_Bullet->myAngle * DTR_RATIO);
				float posY = 2 * sin(object.second->m_Bullet->myAngle * DTR_RATIO);

				object.second->m_Bullet->x += posX;
				object.second->m_Bullet->y += posY;

				m_DrawNode->drawDot(cocos2d::Vec2(object.second->m_Bullet->x, object.second->m_Bullet->y), 1.0f, cocos2d::Color4F(1.0f, 1.0f, 1.0f, 1.0f));

			}
		
		}

		if (m_Utils->getInstance()->distance(object.second->getPosition(), cocos2d::Vec2(object.second->m_Bullet->x, object.second->m_Bullet->y)) > 200.0f) {

			object.second->m_ButtonActive = false;

		}

	}

}

bool GaddBoxScene::onTouchBegan(cocos2d::Touch* touch, cocos2d::Event* _event) {

	// GADDBOX MUST HAVE ACESS TO TOUCHES
	m_TouchBegin = m_Gaddbox->onTouchBegan(touch, _event);

	cocos2d::Rect rect = m_Base->getBoundingBox();

	if (rect.containsPoint(m_TouchBegin)) {
	
	  m_Players.push_back(m_Gaddbox->addObject("circleAgent.png", cocos2d::Vec2(m_Spawn.x + (float(rand() % 10) - 5.0f), m_Spawn.y + (float(rand() % 10) - 5.0f)), 1.0f, 0.2f, 0.2f, 2));
	
	}

	return true;

}

void GaddBoxScene::onTouchMoved(cocos2d::Touch* touch, cocos2d::Event* event) {

	// GADDBOX MUST HAVE ACESS TO TOUCHES MOVED EVENTS
	m_Gaddbox->onTouchMoved(touch, event);

}

void GaddBoxScene::onTouchEnded(cocos2d::Touch* touch, cocos2d::Event* event) {

	// GADDBOX MUST HAVE ACESS TO TOUCHES END EVENTS
	m_Gaddbox->onTouchEnded(touch, event);

}

void GaddBoxScene::onTouchCancelled(cocos2d::Touch* touch, cocos2d::Event* event) {

	m_Gaddbox->onTouchCancelled(touch, event);

}

void GaddBoxScene::initTouch() {

	auto touchListener = cocos2d::EventListenerTouchOneByOne::create();
	touchListener->onTouchBegan = CC_CALLBACK_2(GaddBoxScene::onTouchBegan, this);
	touchListener->onTouchMoved = CC_CALLBACK_2(GaddBoxScene::onTouchMoved, this);
	touchListener->onTouchEnded = CC_CALLBACK_2(GaddBoxScene::onTouchEnded, this);
	touchListener->onTouchCancelled = CC_CALLBACK_2(GaddBoxScene::onTouchCancelled, this);
	_eventDispatcher->addEventListenerWithSceneGraphPriority(touchListener, this);

}

