#include "GameObject.h"

// WE NEED TO INITIALISE THE SPRITE FIRST SO THAT THE PHYSCIS OBJECT CAN GET THE SPORITE SIZE AS SCALING CAN VARY
void GameObject::createSprite(std::string file) {

	initWithSpriteFrameName(file);
	autorelease();
	scheduleUpdate();

}

// WE INITIALISE THE PHYSCIC OBJECT ALONG WITH VARIOUS INSTANIATION OF MEMEBER VARIABLES
bool GameObject::init(int bodyType, float density, float friction, float restitution, bool isStatic) {

	m_Selected = false;
	m_Follow = false;
	m_Flocking = false;
	m_Leader = false;
	m_Reached = false;
	m_ClockWise = false;
	m_Flocking = false;
	m_Cohesion = false;
	m_LinearLeader = true;
	m_Rested = false;
	m_Finished = false;
	m_ButtonActive = false;
	m_Halted = false;
	m_HasTarget = false;

	m_Angle = 0.0f;
	m_Rest = 0.0f;
	m_ToAngle = 0.0f;
	m_Speed = 0.0f;
	m_MaxSpeed = 1.0f;
	m_Steer = 1.0f;
	m_Space = m_SessionData->getInstance()->getSpace();
	m_Distance = m_SessionData->getInstance()->getDistance();

	// FLOCKING WEIGHTING VALUES
	m_AWeight = 1.0f;
	m_CWeight = 1.0f;
	m_SWeight = 1.5f;
	m_LWeight = 1.0f;
	m_VWeight = 1.0f;

	m_Formation = 1;
	m_ID = m_Utils->getInstance()->getCount();

	m_Size = this->getBoundingBox().size;

	m_BodyFixture.density = density;
	m_BodyFixture.friction = friction;
	m_BodyFixture.restitution = restitution;
	m_BodyFixture.filter.categoryBits = OBJECT;
	m_BodyFixture.filter.maskBits = OBJECT;

	m_BodyDef.type = (isStatic == false) ? b2_dynamicBody : b2_staticBody;
	m_BodyDef.userData = this;

	// PICKING OUR SHAPE 
	switch (bodyType) {
	
		// RECTANGLE SHAPE
		case 1: {

			b2Vec2 points[8];
			
			// WE WANT TO CHAMFER EVENLY IF IT IS A RECTANGLE 
			if (m_Size.width > m_Size.height) {
			
				float seg = m_Size.height / 10;
			
			}
			else {

				float seg = m_Size.width / 10;

			}
			
			// ALL POINTS FOR UR BODY
			float seg = m_Size.width / 10;
			points[0].Set((m_Size.width / 2) / 100.0f, ((m_Size.height / 2) - seg) / 100.0f);
			points[1].Set((m_Size.width / 2) / 100.0f, ((-m_Size.height / 2) + seg) / 100.0f);
			points[2].Set(((m_Size.width / 2) - seg) / 100.0f, (-m_Size.height / 2) / 100.0f);
			points[3].Set(((-m_Size.width / 2) + seg) / 100.0f, (-m_Size.height / 2) / 100.0f);
			points[4].Set((-m_Size.width / 2) / 100.0f, ((-m_Size.height / 2) + seg) / 100.0f);
			points[5].Set((-m_Size.width / 2) / 100.0f, ((m_Size.height / 2) - seg) / 100.0f);
			points[6].Set(((-m_Size.width / 2) + seg) / 100.0f, (-m_Size.height / 2) / 100.0f);
			points[7].Set(((m_Size.width / 2) - seg) / 100.0f, (m_Size.height / 2) / 100.0f);
			m_Polygon.Set(points, 8);
			m_BodyFixture.shape = &m_Polygon;
			m_Body = m_SessionData->getInstance()->getB2World()->CreateBody(&m_BodyDef);
			m_Body->CreateFixture(&m_BodyFixture);
			
		}break;

		// CIRCLE SHAPE
		case 2: {

			m_Circle.m_radius = (m_Size.width / 2) / PTM_RATIO;
			m_BodyFixture.shape = &m_Circle;
			m_Body = m_SessionData->getInstance()->getB2World()->CreateBody(&m_BodyDef);
			m_Body->CreateFixture(&m_BodyFixture);

		}break;

		// SQUARE CHAMFERED SHAPE
		case 3: {

			b2PolygonShape m_Shape;
			b2Vec2 points[8];
			float seg = m_Size.width / 10;
			points[0].Set((-seg * 2.0f) / 100.0f, (seg * 5.0f) / 100.0f);
			points[1].Set((seg * 2.0f) / 100.0f, (seg * 5.0f) / 100.0f);
			points[2].Set((seg * 5.0f) / 100.0f, (seg * 2.0f) / 100.0f);
			points[3].Set((seg * 5.0f) / 100.0f, (-seg * 2.0f) / 100.0f);
			points[4].Set((seg * 2.0f) / 100.0f, (-seg * 5.0f) / 100.0f);
			points[5].Set((-seg * 2.0f) / 100.0f, (-seg * 5.0f) / 100.0f);
			points[6].Set((seg * 5.0f) / 100.0f, (-seg * 2.0f) / 100.0f);
			points[7].Set((-seg * 5.0f) / 100.0f, (seg * 2.0f) / 100.0f);
			m_Polygon.Set(points, 8);
			m_BodyFixture.shape = &m_Polygon;
			m_Body = m_SessionData->getInstance()->getB2World()->CreateBody(&m_BodyDef);
			m_Body->CreateFixture(&m_BodyFixture);

		}break;

		// DEFAULT IS CIRCLE
		default: {

			m_Circle.m_radius = (m_Size.width / 2) / 100.0f;
			m_BodyFixture.shape = &m_Circle;
			m_Body = m_SessionData->getInstance()->getB2World()->CreateBody(&m_BodyDef);
			m_Body->CreateFixture(&m_BodyFixture);

		}break;
	
	}

	// SET TRANSFORM VALUE - NOTE HOW WE ALWAYS HAVE TO USE THE RADIAN TO DEGREE RATIO
	m_Body->SetTransform(b2Vec2(this->getPosition().x / PTM_RATIO, this->getPosition().y / PTM_RATIO), -this->getRotation()* RTD_RATIO);

	m_Body->SetLinearDamping(1.0f);
	m_Body->SetAngularDamping(0.5f);

	cocos2d::Sprite* mySprite = cocos2d::Sprite::create("mysprite.png");

	m_Bullet = new MyBullet();

    return true;

}

// GAME OBJECT UPDATE FUCNTION HAS A LOT OF API RESPONSIBILITY
void GameObject::update() {

	// IF THE GAME OBJECTY IS A LEADER
	if (m_Leading) {

		m_Halted = false;
	
		int count = m_GameObjects.size(); // LEADER FOLLOWER SIZE

		int m_Count = 0; // COUNT = ZERO - WE NEED TO C OUNT TO FILL OUT THE LAST ROW IF THE FORMATION HAS A UNEVEN NUMBER

		m_Position = getPosition();

		switch (m_Formation) {

			// FLOCKING FORMATION - THE LEADER DOES NOT DO MUCH WORK HERE - THE FOLLOWER DOES
			case 1 : {

				m_Position = this->getPosition();

				m_Angle = m_Body->GetAngle() * RTD_RATIO;
				m_ToAngle = atan2(m_Target.y - m_Position.y, m_Target.x - m_Position.x) * RTD_RATIO;

				m_ToAngle = normalise(m_ToAngle);
				m_Angle = normalise(m_Angle);

				m_Speed = setSpeed();

				float speed = (distance(m_Position, m_Target) / distance(getCenterPoint(m_GameObjects), m_Target)) * m_Speed;

				if (m_LinearLeader) { m_Body->SetTransform(m_Body->GetPosition(), m_ToAngle / RTD_RATIO); }
				else { rotate(); }

				if (distance(m_Position, m_Target) > 10.0f) {

					float32 vecX = speed * cos(m_Body->GetAngle());
					float32 vecY = speed * sin(m_Body->GetAngle());

					m_Body->SetLinearVelocity(b2Vec2(vecX, vecY));

				}

				else {

					m_Body->SetLinearVelocity(b2Vec2(0, 0));
					m_Leading = false;

					for (auto object : m_GameObjects) {

						object.second->setFollow(false);
						object.second->setLeader(nullptr);						

					}

				}

			}break;

			// SQUARE FORMATION
			case 2 : {

				if (!m_Nodes.empty()) { clearNodes(); }

				int x;
				int y;

				// VARIOUS CALCULATIONS TO GET READY
				int row = round(std::sqrt(float(count + 1))); // IF WE GET SQUARE ROOT , WE GET ROUGH ROW AND COLUMN FOR OUR SQUARE SHAPE
				if (count == 6) { row = 3; }
				int lastRow = count % row; // LAST ROW ARE THE LEFT OVER GAME OBJECTS IF THE CONT IS UNEVEN FOR A PERFECT SQUARE
				float space = m_Space / 1.5f;
				float offset = float(row - 1) * (space / 2);
				float spriteOffset = 2.5f;

				m_GameObjects_Iter = m_GameObjects.begin();

				// DEALING WITH A SINGLE FOLLOWER
				if (count == 1) {

					cocos2d::Node* node = cocos2d::Node::create();
					node->setPosition(cocos2d::Vec2(-space + offset + spriteOffset, 0));
					node->setTag(0);
					m_Nodes.push_back(node);
					this->addChild(node);

					(*m_GameObjects_Iter).second->setTarget(cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2));


				}
				
				// DEALING WITH 2 OR MORE
				else {

					// CREATE OUR NODES AND ASSING THEM TO A FOLLOWER
					for (x = 0; x < row && m_Count < count - lastRow; x++) {

						for (int y = 0; y < row && m_Count < count - lastRow; y++) {

							cocos2d::Node* node = cocos2d::Node::create();
							this->addChild(node);
							node->setPosition(cocos2d::Vec2((-(x * space) + offset) + spriteOffset, ((y * space) - offset) + spriteOffset));

							cocos2d::Vec2 position = node->getPosition();

							if (distance(node->getPosition(), cocos2d::Vec2(0.0f, 0.0f)) < 5.0f) {

								node->setPosition(cocos2d::Vec2(node->getPosition().x - 10.0f, node->getPosition().y));
							
							}

							node->setTag(0);
							m_Nodes.push_back(node);
							(*m_GameObjects_Iter).second->setTarget(cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2));
							m_GameObjects_Iter++;

							m_Count++;

						}

					}

					// LAST ROW NEEDS CREATING AND ASSIGNING - THIS IS DUE TO HAVING A UNEVEN NUMBER OF FOLLOWERS, 
					// EXAMPLE IF WE HAVE 18 FOLLOWERS, WE HAVE TWO LEFT AT BACK ROW
					if (m_Count < count) {

						float back = ((float(row) - float(lastRow)) * space) / 2;

						for (y = 0; y < lastRow; y++) {

							cocos2d::Node* node = cocos2d::Node::create();
							node->setPosition(cocos2d::Vec2((-(float(x) * space) + offset) + spriteOffset, (((float(y) * space) + back) - offset) + spriteOffset));
							node->setTag(0);
							m_Nodes.push_back(node);
							this->addChild(node, 1);

							(*m_GameObjects_Iter).second->setTarget(cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2));
							m_GameObjects_Iter++;

							m_Count++;

						}

					}

				}

				// POSITIONAL CALCULATIONS
				m_Position = this->getPosition();
				m_ToAngle = atan2(m_Target.y - m_Position.y, m_Target.x - m_Position.x) * RTD_RATIO;
				m_Speed = setSpeed();
				m_Body->SetTransform(m_Body->GetPosition(), m_ToAngle / RTD_RATIO);

				// SPEED CALCULATIONS
				if (distance(m_Position, m_Target) > 1.0f) {

					float32 vecX = m_Speed * cos(m_Body->GetAngle());
					float32 vecY = m_Speed * sin(m_Body->GetAngle());

					m_Body->SetLinearVelocity(b2Vec2(vecX, vecY));

				} 
				
				else {

					float angle = m_Rest * DTR_RATIO;

					m_Body->SetTransform(m_Body->GetPosition(), m_Rest * DTR_RATIO);

					m_Rested = true;

					m_Body->SetLinearVelocity(b2Vec2(0.0f, 0.0f));

					if (m_Finished) {

						m_Leading = false;
						m_Rested = false;
						m_Finished = false;

						for (auto object : m_GameObjects) {

							object.second->setFollow(false);
							object.second->setLeader(nullptr);

						}
					
					}

					else {

						bool childrenFinished = true;

						for (auto object : m_GameObjects) {

							if (!object.second->getFinished()) { childrenFinished = false; };

						}

						if (childrenFinished) { 
							
							m_Finished = true; 
						
						}

					}

				}

				//for (auto node : m_Nodes) {
					//cocos2d::Vec2 nodePos = node->getPosition();
					//cocos2d::Vec2 parentPos = this->getPosition();
					//cocos2d::Vec2 transPos = cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2);
					//m_SessionData->getInstance()->getDrawNode()->drawCircle(cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2), 5, 0, 12, false, cocos2d::Color4F(1.0f, 0.0f, 0.0f, 1.0f));
				//}

			}break;

			// CIRCLE FORMATION
			case 3 : {

				if (!m_Nodes.empty()) { clearNodes(); }

				m_GameObjects_Iter = m_GameObjects.begin();

				int layer;
				int set;
				int counter;

				int y;
				int x;

				// SORTING THE ROWS IN OUR CIRLE FORMATION
				if (count <= 8) {

					layer = 1;
					set = 8;
					counter = 0;

					y = 0;
					x = 0;
					
					float angleSeg = 360.f / float(count);

					for (x = 0; x < count; x++) {

						float angle = (angleSeg * float(x)) - 180.0f;

						float posX = m_Space * cos(angle * DTR_RATIO);
						float posY = m_Space * sin(angle * DTR_RATIO);


						cocos2d::Node* node = cocos2d::Node::create();
						this->addChild(node);
						node->setPosition(cocos2d::Vec2(posX, posY));
						node->setTag(0);
						m_Nodes.push_back(node);
						(*m_GameObjects_Iter).second->setTarget(cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2));
						m_GameObjects_Iter++;

						counter++;

					}
					
				}
				
				else if (count <= 24) {

					layer = 2;
					set = count / 3;
					counter = 0;
					
					y = 0;
					x = 0;

					
				}
				
				else if (count <= 32) {

					layer = 3;
					set = count / 3;
					counter = 0;

					y = 0;
					x = 0;

				}
				
				else if (count <= 64) {

					layer = 4;
					set = count / 3;
					counter = 0;

					y = 0;
					x = 0;

				}
				
				for (y = 1; y <= layer; y++) {

					if (y == layer) { set = count - counter; } // FIT ALL REMAINING INTO LAST RING

					for (x = 0; x < set; x++) {

						if (counter < count) {

							float angleSegment = ((360.0f / float(set)) * float(x)) - 180.0f;

							float posX = (m_Space * float(y)) * cos(angleSegment * DTR_RATIO);
							float posY = (m_Space * float(y)) * sin(angleSegment * DTR_RATIO);

							cocos2d::Node* node = cocos2d::Node::create();
							this->addChild(node);
							node->setPosition(cocos2d::Vec2(posX, posY));
							node->setTag(0);
							m_Nodes.push_back(node);
							(*m_GameObjects_Iter).second->setTarget(cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2));
							m_GameObjects_Iter++;

							counter++;

						}

					}
					set *= 2;

				}

				m_Position = this->getPosition();
				m_ToAngle = atan2(m_Target.y - m_Position.y, m_Target.x - m_Position.x) * RTD_RATIO;
				m_Speed = setSpeed();
				m_Body->SetTransform(m_Body->GetPosition(), m_ToAngle / RTD_RATIO);

				if (distance(m_Position, m_Target) > 1.0f) {

					float32 vecX = m_Speed * cos(m_Body->GetAngle());
					float32 vecY = m_Speed * sin(m_Body->GetAngle());

					m_Body->SetLinearVelocity(b2Vec2(vecX, vecY));

				}

				else {

					float angle = m_Rest * DTR_RATIO;

					m_Body->SetTransform(m_Body->GetPosition(), m_Rest * DTR_RATIO);

					m_Rested = true;

					m_Body->SetLinearVelocity(b2Vec2(0.0f, 0.0f));

					if (m_Finished) {

						m_Leading = false;
						m_Rested = false;
						m_Finished = false;

						for (auto object : m_GameObjects) {

							object.second->setFollow(false);
							object.second->setLeader(nullptr);

						}

					}

					else {

						bool childrenFinished = true;

						for (auto object : m_GameObjects) {

							if (!object.second->getFinished()) { childrenFinished = false; };

						}

						if (childrenFinished) {

							m_Finished = true;

						}

					}

				}

				//for (auto node : m_Nodes) {
					//cocos2d::Vec2 nodePos = node->getPosition();
					//cocos2d::Vec2 parentPos = this->getPosition();
					//cocos2d::Vec2 transPos = cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2);
					//m_SessionData->getInstance()->getDrawNode()->drawCircle(cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2), 5, 0, 12, false, cocos2d::Color4F(1.0f, 0.0f, 0.0f, 1.0f));
				//}

			}break;

			// LINE FORMATION
			case 4 : {

				if (!m_Nodes.empty()) { clearNodes(); }

				int x;
				int y;

				// OUR LINE IS A 2 BY 1 SHAPE

				int segment = count / 3;
				int leftover = count - (segment * 3);

				int row = 3;
				int column = segment;

				float space = m_Space / 1.5f;

				float offsetX = float(row - 1) * (space / 2);
				float offsetY = float(column - 1) * (space / 2);

				float spriteOffset = 2.5f;

				m_GameObjects_Iter = m_GameObjects.begin();

				if (count == 1) {

					cocos2d::Node* node = cocos2d::Node::create();
					node->setPosition(cocos2d::Vec2(-space + offsetX + spriteOffset, 0));
					node->setTag(0);
					m_Nodes.push_back(node);
					this->addChild(node);

					(*m_GameObjects_Iter).second->setTarget(cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2));


				}

				else {

					// ASSINGING AND CREATING NODES TO OUR FOLLOWERS
					for (x = 0; x < row; x++) {

						for (int y = 0; y < column; y++) {

							cocos2d::Node* node = cocos2d::Node::create();
							this->addChild(node);
							node->setPosition(cocos2d::Vec2((-(x * space) + offsetX) + spriteOffset, ((y * space) - offsetY) + spriteOffset));
							cocos2d::Vec2 position = node->getPosition();
							if (distance(node->getPosition(), cocos2d::Vec2(0.0f, 0.0f)) < 5.0f) { node->setPosition(cocos2d::Vec2(node->getPosition().x - 10.0f, node->getPosition().y)); }
							node->setTag(0);
							m_Nodes.push_back(node);
							(*m_GameObjects_Iter).second->setTarget(cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2));
							m_GameObjects_Iter++;

							m_Count++;

						}

					}

					// DEALING WITH LAST ROW LEFTOVERS 
					if (m_Count < count) {

						x++;

						float back = ((float(row) - float(leftover)) * space) / 2;

						for (y = 0; y < leftover; y++) {

							cocos2d::Node* node = cocos2d::Node::create();
							node->setPosition(cocos2d::Vec2((-(float(x) * space) + offsetX) + spriteOffset, (((float(y) * space) + back) - offsetY) + spriteOffset));
							node->setTag(0);
							m_Nodes.push_back(node);
							this->addChild(node, 1);

							(*m_GameObjects_Iter).second->setTarget(cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2));
							m_GameObjects_Iter++;

							m_Count++;

						}

					}

				}

				m_Position = this->getPosition();
				m_ToAngle = atan2(m_Target.y - m_Position.y, m_Target.x - m_Position.x) * RTD_RATIO;
				m_Speed = setSpeed();
				m_Body->SetTransform(m_Body->GetPosition(), m_ToAngle / RTD_RATIO);

				if (distance(m_Position, m_Target) > 1.0f) {

					float32 vecX = m_Speed * cos(m_Body->GetAngle());
					float32 vecY = m_Speed * sin(m_Body->GetAngle());

					m_Body->SetLinearVelocity(b2Vec2(vecX, vecY));

				}

				else {

					float angle = m_Rest * DTR_RATIO;

					m_Body->SetTransform(m_Body->GetPosition(), m_Rest * DTR_RATIO);

					m_Rested = true;

					m_Body->SetLinearVelocity(b2Vec2(0.0f, 0.0f));

					if (m_Finished) {

						m_Leading = false;
						m_Rested = false;
						m_Finished = false;

						for (auto object : m_GameObjects) {

							object.second->setFollow(false);
							object.second->setLeader(nullptr);

						}

					}

					else {

						bool childrenFinished = true;

						for (auto object : m_GameObjects) {

							if (!object.second->getFinished()) { childrenFinished = false; };

						}

						if (childrenFinished) {

							m_Finished = true;

						}

					}

				}

				//for (auto node : m_Nodes) {
					//cocos2d::Vec2 nodePos = node->getPosition();
					//cocos2d::Vec2 parentPos = this->getPosition();
					//cocos2d::Vec2 transPos = cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2);
					//m_SessionData->getInstance()->getDrawNode()->drawCircle(cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2), 5, 0, 12, false, cocos2d::Color4F(1.0f, 0.0f, 0.0f, 1.0f));
				//}

			}break;

			default: {

				if (!m_Nodes.empty()) { clearNodes(); }

				int x;
				int y;

				int row = round(std::sqrt(float(count + 1)));

				if (count == 6) { row = 3; }

				int lastRow = count % row;

				float space = m_Space / 1.5f;

				float offset = float(row - 1) * (space / 2);

				float spriteOffset = 2.5f;

				m_GameObjects_Iter = m_GameObjects.begin();

				if (count == 1) {

					cocos2d::Node* node = cocos2d::Node::create();
					node->setPosition(cocos2d::Vec2(-space + offset + spriteOffset, 0));
					node->setTag(0);
					m_Nodes.push_back(node);
					this->addChild(node);

					(*m_GameObjects_Iter).second->setTarget(cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2));


				}

				else {

					for (x = 0; x < row && m_Count < count - lastRow; x++) {

						for (int y = 0; y < row && m_Count < count - lastRow; y++) {

							cocos2d::Node* node = cocos2d::Node::create();
							this->addChild(node);
							node->setPosition(cocos2d::Vec2((-(x * space) + offset) + spriteOffset, ((y * space) - offset) + spriteOffset));

							cocos2d::Vec2 position = node->getPosition();

							if (distance(node->getPosition(), cocos2d::Vec2(0.0f, 0.0f)) < 5.0f) {

								node->setPosition(cocos2d::Vec2(node->getPosition().x - 10.0f, node->getPosition().y));

							}

							node->setTag(0);
							m_Nodes.push_back(node);
							(*m_GameObjects_Iter).second->setTarget(cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2));
							m_GameObjects_Iter++;

							m_Count++;

						}

					}

					if (m_Count < count) {

						float back = ((float(row) - float(lastRow)) * space) / 2;

						for (y = 0; y < lastRow; y++) {

							cocos2d::Node* node = cocos2d::Node::create();
							node->setPosition(cocos2d::Vec2((-(float(x) * space) + offset) + spriteOffset, (((float(y) * space) + back) - offset) + spriteOffset));
							node->setTag(0);
							m_Nodes.push_back(node);
							this->addChild(node, 1);

							(*m_GameObjects_Iter).second->setTarget(cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2));
							m_GameObjects_Iter++;

							m_Count++;

						}

					}

				}

				m_Position = this->getPosition();
				m_ToAngle = atan2(m_Target.y - m_Position.y, m_Target.x - m_Position.x) * RTD_RATIO;
				m_Speed = setSpeed();
				m_Body->SetTransform(m_Body->GetPosition(), m_ToAngle / RTD_RATIO);

				if (distance(m_Position, m_Target) > 1.0f) {

					float32 vecX = m_Speed * cos(m_Body->GetAngle());
					float32 vecY = m_Speed * sin(m_Body->GetAngle());

					m_Body->SetLinearVelocity(b2Vec2(vecX, vecY));

				}

				else {

					float angle = m_Rest * DTR_RATIO;

					m_Body->SetTransform(m_Body->GetPosition(), m_Rest * DTR_RATIO);

					m_Rested = true;

					m_Body->SetLinearVelocity(b2Vec2(0.0f, 0.0f));

					if (m_Finished) {

						m_Leading = false;
						m_Rested = false;
						m_Finished = false;

						for (auto object : m_GameObjects) {

							object.second->setFollow(false);
							object.second->setLeader(nullptr);

						}

					}

					else {

						bool childrenFinished = true;

						for (auto object : m_GameObjects) {

							if (!object.second->getFinished()) { childrenFinished = false; };

						}

						if (childrenFinished) {

							m_Finished = true;

						}

					}

				}

				//for (auto node : m_Nodes) {
				//cocos2d::Vec2 nodePos = node->getPosition();
				//cocos2d::Vec2 parentPos = this->getPosition();
				//cocos2d::Vec2 transPos = cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2);
				//m_SessionData->getInstance()->getDrawNode()->drawCircle(cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2), 5, 0, 12, false, cocos2d::Color4F(1.0f, 0.0f, 0.0f, 1.0f));
				//}

			}break;

		}
	
	}
	
	// IF THE GAME OBJECT IS FOLLOWING A LEADER
	else if (m_Follow) {

		m_Halted = false;

		switch (m_Leader->getFormation()) {

		case 1: {

			m_Position = getPosition();
			m_Target = m_Leader->getPosition();

			std::vector<cocos2d::Vec2> vectors;

			cocos2d::Vec2 alignment_ = alignment(); vectors.push_back(alignment());
			cocos2d::Vec2 cohesion_ = cohesion(); vectors.push_back(cohesion());
			cocos2d::Vec2 seperation_ = seperation(); vectors.push_back(seperation());
			cocos2d::Vec2 vector_ = vector(); vectors.push_back(vector());
			cocos2d::Vec2 leader_ = leader(); vectors.push_back(leader());

			cocos2d::Vec2 result = cocos2d::Vec2(0.0f, 0.0f);

			for (auto vector : vectors) {
			
				result.x += vector.x;
				result.y += vector.y;
			
			}

			result = unitVector(result);

			m_ToAngle = atan2(result.y, result.x) * RTD_RATIO;
			m_Angle = m_Body->GetAngle() * RTD_RATIO;

			m_ToAngle = normalise(m_ToAngle);
			m_Angle = normalise(m_Angle);

			rotate();

			m_Speed = (1.0f - (1 / distance(m_Target, m_Position))) * (m_MaxSpeed);

			float vecX = m_Speed * cos(m_Body->GetAngle());
			float vecY = m_Speed * sin(m_Body->GetAngle());

			m_Body->SetLinearVelocity(b2Vec2(float32(vecX), float32(vecY)));

		}break;

		case 2: {

			m_Position = getPosition();

			m_ToAngle = atan2(m_Target.y - m_Position.y, m_Target.x - m_Position.x);

			m_Body->SetTransform(m_Body->GetPosition(), m_ToAngle);

			float speed = 1 / distance(m_Target, m_Position);

			m_Speed = (1.0f - (1 / distance(m_Target, m_Position))) * (2);

			float vecX = m_Speed * cos(m_Body->GetAngle());
			float vecY = m_Speed * sin(m_Body->GetAngle());

			m_Body->SetLinearVelocity(b2Vec2(float32(vecX), float32(vecY)));

			if (m_Leader->getRested()) {
			
				if (distance(m_Target, m_Position) > 0.1f) {

					m_Finished = true;
					m_Body->SetLinearVelocity(b2Vec2(0.0f, 0.0f));
					m_Body->SetAngularVelocity(0.0f);

				} else { m_Finished = false; }
			
			} else { m_Finished = false; }

		}break;

		case 3: {

			m_Position = getPosition();

			m_ToAngle = atan2(m_Target.y - m_Position.y, m_Target.x - m_Position.x);

			m_Body->SetTransform(m_Body->GetPosition(), m_ToAngle);

			float speed = 1 / distance(m_Target, m_Position);

			m_Speed = (1.0f - (1 / distance(m_Target, m_Position))) * (2);

			float vecX = m_Speed * cos(m_Body->GetAngle());
			float vecY = m_Speed * sin(m_Body->GetAngle());

			m_Body->SetLinearVelocity(b2Vec2(float32(vecX), float32(vecY)));

			if (m_Leader->getRested()) {

				if (distance(m_Target, m_Position) > 0.1f) {

					m_Finished = true;
					m_Body->SetLinearVelocity(b2Vec2(0.0f, 0.0f));
					m_Body->SetAngularVelocity(0.0f);

				}
				else { m_Finished = false; }

			}
			else { m_Finished = false; }

		}break;

		case 4: {

			m_Position = getPosition();

			m_ToAngle = atan2(m_Target.y - m_Position.y, m_Target.x - m_Position.x);

			m_Body->SetTransform(m_Body->GetPosition(), m_ToAngle);

			float speed = 1 / distance(m_Target, m_Position);

			m_Speed = (1.0f - (1 / distance(m_Target, m_Position))) * (2);

			float vecX = m_Speed * cos(m_Body->GetAngle());
			float vecY = m_Speed * sin(m_Body->GetAngle());

			m_Body->SetLinearVelocity(b2Vec2(float32(vecX), float32(vecY)));

			if (m_Leader->getRested()) {

				if (distance(m_Target, m_Position) > 0.1f) {

					m_Finished = true;
					m_Body->SetLinearVelocity(b2Vec2(0.0f, 0.0f));
					m_Body->SetAngularVelocity(0.0f);

				}
				else { m_Finished = false; }

			}
			else { m_Finished = false; }

		}break;

		default: {

			m_Position = getPosition();
			m_Target = m_Leader->getPosition();

			std::vector<cocos2d::Vec2> vectors;

			cocos2d::Vec2 alignment_ = alignment(); vectors.push_back(alignment());
			cocos2d::Vec2 cohesion_ = cohesion(); vectors.push_back(cohesion());
			cocos2d::Vec2 seperation_ = seperation(); vectors.push_back(seperation());
			cocos2d::Vec2 vector_ = vector(); vectors.push_back(vector());
			cocos2d::Vec2 leader_ = leader(); vectors.push_back(leader());

			cocos2d::Vec2 result = cocos2d::Vec2(0.0f, 0.0f);

			for (auto vector : vectors) {

				result.x += vector.x;
				result.y += vector.y;

			}

			result = unitVector(result);

			m_ToAngle = atan2(result.y, result.x) * RTD_RATIO;
			m_Angle = m_Body->GetAngle() * RTD_RATIO;

			m_ToAngle = normalise(m_ToAngle);
			m_Angle = normalise(m_Angle);

			rotate();

			m_Speed = (1.0f - (1 / distance(m_Target, m_Position))) * (m_MaxSpeed);

			float vecX = m_Speed * cos(m_Body->GetAngle());
			float vecY = m_Speed * sin(m_Body->GetAngle());

			m_Body->SetLinearVelocity(b2Vec2(float32(vecX), float32(vecY)));

		}break;

		}

	}
	
	else if (m_HasTarget) {
	
		m_Position = getPosition();

		m_ToAngle = atan2(m_Target.y - m_Position.y, m_Target.x - m_Position.x) * RTD_RATIO;
		m_Angle = m_Body->GetAngle() * RTD_RATIO;

		m_ToAngle = normalise(m_ToAngle);
		m_Angle = normalise(m_Angle);

		rotate();

		m_Speed = (1.0f - (1 / distance(m_Target, m_Position))) * (m_MaxSpeed);

		float vecX = m_Speed * cos(m_Body->GetAngle());
		float vecY = m_Speed * sin(m_Body->GetAngle());

		m_Body->SetLinearVelocity(b2Vec2(float32(vecX), float32(vecY)));
	
	}

	// ELSE IF THE OBJECT IS NOT FREE, IT FORCE IS ZERO
	else {

		m_Halted = false;

		m_Body->SetLinearVelocity(b2Vec2(0, 0));
		m_Body->SetAngularVelocity(0.0f);

	}

	// IF AN OBJECT IS FREE THE DEVELOPER CAN USE IT FOR YUR OWN REASONS

	this->setPosition(cocos2d::Vec2(m_Body->GetPosition().x * PTM_RATIO, m_Body->GetPosition().y * PTM_RATIO));
	this->setRotation(-m_Body->GetAngle() * RTD_RATIO);

}

// DESTROY THE PHYSICS BODY
void GameObject::clean() {

	m_SessionData->getInstance()->getB2World()->DestroyBody(m_Body);

}

// CLEAR ALL NODES
void GameObject::clearNodes() {

	for (m_Nodes_Iter = m_Nodes.begin(); m_Nodes_Iter != m_Nodes.end();) {

		this->removeChild((*m_Nodes_Iter), true);
		m_Nodes_Iter = m_Nodes.erase(m_Nodes_Iter);

	}

}

// THIS WAS ACTUALLY A TRICKY LITTLE PIECE OF CODE
// WE WANT TO GET THE DIFFERENCE BETWEEN TWO ANGLES
// EXAMPLE   ANGLE 1 = 170    ANGLE TWO = -170    ANSWER = 20 DIFFERENCE
// THIS IS BECAUSE WE ARE DEALING WITH THE RANGE OF 180 - -180 WHERE -180 = 180
float GameObject::getAngleDiff(float x, float y) {

	if (x == y) {

		return 0.0f;

	}
	
	else if (x == 0.0f) {

		if (y < 0.0f) {

			m_ClockWise = true;
			return -(y);

		}
		
		else {

			m_ClockWise = false;
			return y;

		}

	}
	
	else if (x < 0.0f) {

		if (y == 0.0f) {

			m_ClockWise = false;
			return -x;

		}
		
		else if (y < 0.0f) {

			if (x < y) {

				m_ClockWise = false;
				return -x - (-y);

			}
			else {

				m_ClockWise = true;
				return -y - (-x);

			}

		}
		
		else if (y > 0.0f) {

			if (-x + y < 180) {

				m_ClockWise = false;
				return -x + y;

			}
			else {

				m_ClockWise = true;
				return (180 - (-x)) + (180 - y);

			}

		}

	}
	
	else if (x > 0.0f) {

		if (y == 0.0f) {

			m_ClockWise = true;
			return x;

		}
		
		else if (y > 0.0f) {

			if (x > y) {

				m_ClockWise = true;
				return x - y;

			}
			else {

				m_ClockWise = false;
				return y - x;

			}

		}
		
		else if (y < 0.0f) {

			if (x + (-y) < 180) {

				m_ClockWise = true;
				return x + (-y);

			}
			else {

				m_ClockWise = false;
				return (180 - (-y)) + (180 - x);

			}

		}

	}

	return 0.0f;

}

// WE USE THE CLOCKWISE BOOLEAN TO ROTATE BY A GIVEN VALUE DEPENDING ON THE DIFFERENT BETWEEN TARGET ANGLE AND CURRENT ANGLE
void GameObject::rotate() {

	float difference = getAngleDiff(m_Angle, m_ToAngle);

	if (difference <= 1.0f) {
	
		m_Angle = m_ToAngle;
		m_Body->SetAngularVelocity(float32(0));
	
	}

	else {

		m_Steer = difference / 180;

		m_Steer *= 10;

		if (m_ClockWise) {

			m_Body->SetAngularVelocity(float32(-m_Steer));

		}

		else if (!m_ClockWise) {

			m_Body->SetAngularVelocity(float32(m_Steer));

		}

	}

}

// WE NEED TO BE ABLE NORMALISE ANY ANGLE SO THAT WE ARE ALWAYS DEALING WITH 180 TO -180 ONLY
float GameObject::normalise(float angle) {


	if (angle > 0.0f) {

		angle = std::fmod(angle, 360.0f);

		if (angle > 180) {

			return (angle - 180) - 180;

		}

	}
	else if (angle < 0.0f) {

		angle = std::fmod(angle, -360.0f);

		if (angle < -180) {

			return -((-(angle)-180) - 180);

		}

	}

	return angle;


}

// DISTANCE BETWEEN TWO POINTS
float GameObject::distance(cocos2d::Vec2 position, cocos2d::Vec2 target) {

	float x_diff = position.x - target.x;
	float y_diff = position.y - target.y;

	float diff = std::sqrt(x_diff * x_diff + y_diff * y_diff);

	return diff;

}

// SIMPLE LOGARITHMIC SPEED CALCULATOR
float GameObject::setSpeed() {

	return 1.0f - (1 / distance(m_Target, m_Position));

}

// GET AVERAGE CENTER POINT OF E VECTOR OF OBJECTS
cocos2d::Vec2 GameObject::getCenterPoint(std::map<int, GameObject*> objects) {

	if (objects.size() > 0) {

		float x = 0.0f;
		float y = 0.0f;

		for (auto object : objects) {

			x += object.second->getPosition().x;
			y += object.second->getPosition().y;

		}

		x = x / objects.size();
		y = y / objects.size();

		return cocos2d::Vec2(x, y);
	
	}

	return cocos2d::Vec2(0.0f, 0.0f);

}

// TELL OBJECT STOP ALL ITS FOLLOWING ACTIVITIES
void GameObject::stopFollow() {

	m_Leader->getGameObjects().erase(m_ID);
	m_Follow = false;
	m_Leader = nullptr;

}

// FLOCKING ALLIGNMENT FUCNTION - ALL FUNCTIONS ARE WEIGHTED
// WE RETURN ALL AVERAGE VECTOR FROM NEIGHBOURS
cocos2d::Vec2 GameObject::alignment() {

	float averageX = 0.0f;
	float averageY = 0.0f;

	float count = 0.0f;

	for (auto object : m_Leader->getGameObjects()) {

		if (object.second != this) {

			if (distance(m_Position, object.second->getPosition()) < m_Space) {

				averageX += object.second->getVector().x;
				averageY += object.second->getVector().y;
				count += 1.0f;

			}

		}

	}

	if (count == 0.0f) { return cocos2d::Vec2(0.0f, 0.0f); } // RETURN EMPTY VECTOR IF NO COHESION OBJECTS

	return cocos2d::Vec2((averageX / count) * m_AWeight, (averageY / count) * m_AWeight);

}

// WE RETURN VECTOR THAT STEERS TOWARDS NEIGHBOURS AVERAGE CENTER POINT
cocos2d::Vec2 GameObject::cohesion() {

	std::map<int, GameObject*> objects;

	m_Cohesion = false;

	for (auto object : m_Leader->getGameObjects()) {

		if (object.second != this) {

			if (distance(m_Position, object.second->getPosition()) <  m_Space) {

				objects.insert(object);

			}

		}

	}

	if (objects.empty()) { return cocos2d::Vec2(0.0f, 0.0f); } // RETURN EMPTY VECTOR IF NO COHESION OBJECTS

	m_Cohesion = true;

	cocos2d::Vec2 center = getCenterPoint(objects);

	return cocos2d::Vec2((center.x - m_Position.x) * m_CWeight, (center.y - m_Position.y) * m_CWeight);

}

// WE ADD UP ALL VECTORS THAT STEER US TOWARDS OUR NEIGHBOURS. eVERY VECTOR DISTANCE FOR EACH NEIGHBOURS IS ADDED UP, WE RETURN THE TOTAL SUM
cocos2d::Vec2 GameObject::seperation() {

	float averageX = 0.0f;
	float averageY = 0.0f;

	float count = 0.0f;

	for (auto object : m_Leader->getGameObjects()) {

		if (object.second != this) {

			if (distance(m_Position, object.second->getPosition()) < m_Space) {

				averageX += object.second->getPosition().x - this->getPosition().x;
				averageY += object.second->getPosition().y - this->getPosition().y;
				count += 1.0f;

			}

		}

	}

	if (count == 0.0f) { return cocos2d::Vec2(0, 0); }

	averageX *= -1;
	averageY *= -1;

	return cocos2d::Vec2(averageX * m_SWeight, averageY * m_SWeight);

}

// WE RETURN THE VECTOR THAT STEERS US TOWARDS THE LEADER
cocos2d::Vec2 GameObject::leader() {

	m_Position = this->getPosition();
	m_Target = m_Leader->getPosition();

	return cocos2d::Vec2((m_Target.x - m_Position.x) * m_LWeight, (m_Target.y - m_Position.y) * m_LWeight);
}

// WE RETURN THE VECTOR THAT STEERS IN ALLIGNMENT WITH OUR LEADER
cocos2d::Vec2 GameObject::vector() {

	return cocos2d::Vec2((m_Body->GetLinearVelocity().x * RTD_RATIO) * m_VWeight, (m_Body->GetLinearVelocity().y * RTD_RATIO) * m_VWeight);

}

// CONVERT A VECTOR TO A UNIT VECTOR
cocos2d::Vec2 GameObject::unitVector(cocos2d::Vec2 vector) {

	if (vector.isZero()) { return cocos2d::Vec2(0.0f, 0.0f); }

	float hypo = sqrt(pow(vector.x, 2) + pow(vector.y, 2));

	float vecX = vector.x / hypo;
	float vecY = vector.y / hypo;

	return cocos2d::Vec2(vecX, vecY);

}

// GET ALL NEIGHBOURS WITHINT A DISTANCE
cocos2d::Vec2 GameObject::neighbour(std::map<int, GameObject*>& objects) {

	cocos2d::Vec2 neighbour;

	neighbour = objects.begin()->second->getPosition();

	float distance = sqrt(pow(objects.begin()->second->getPosition().x - this->getPosition().x, 2) + pow(objects.begin()->second->getPosition().y - this->getPosition().y, 2));

	for (auto object : m_Leader->getGameObjects()) {

		if (object.first != objects.begin()->first && object.first != m_ID) {

			float newDistance = sqrt(pow(object.second->getPosition().x - this->getPosition().x, 2) + pow(object.second->getPosition().y - this->getPosition().y, 2));
		
			if (newDistance < distance) {
			
				distance = newDistance;
				neighbour = object.second->getPosition();
			
			}


		
		}

	}

	return neighbour;
}

// RETURN NODES POSITION IN WORLD SPACE
cocos2d::Vec2 GameObject::worldSpace(cocos2d::Node* node) {

	return cocos2d::Vec2(node->getParent()->convertToWorldSpace(node->getPosition()).x - m_SessionData->getInstance()->getWinWidth() / 2, node->getParent()->convertToWorldSpace(node->getPosition()).y - m_SessionData->getInstance()->getWinHeight() / 2);
}

