// THIS IS THE GAME OBJECT CLASS 

#ifndef GADDBOX_GAMEOBJECT_H
#define GADDBOX_GAMEOBJECT_H

#define PTM_RATIO 100.0f
#define DTR_RATIO 0.0174532925199432957f
#define RTD_RATIO 57.295779513082320876f

#include "cocos2d.h"
#include "MyBullet.h"

#include "SessionData.h"
#include "Utils.h"

class GameObject : public cocos2d::Sprite {

	public:

		void createSprite(std::string file);
		bool init(int bodyType, float density, float friction, float restitution, bool isStatic);
		void update();
		void clean();

		std::map<int, GameObject*>& getGameObjects() { return m_GameObjects; };

		cocos2d::Size getSize() { return m_Size; };

		float getSpeed() { return m_Speed; };
		float getRestAngle() { return m_Rest; };
		
		cocos2d::Vec2 getTarget() { return m_Target; };
		cocos2d::Vec2 getVector() { return cocos2d::Vec2(m_Body->GetLinearVelocity().x, m_Body->GetLinearVelocity().y); }
		cocos2d::Vec2 getCenterPoint(std::map<int, GameObject*> objects);
		cocos2d::Vec2 worldSpace(cocos2d::Node* node);

		int getFormation() { return m_Formation; };
		int getID() { return m_ID; };

		void setSpace(float space) { m_Space = space; };
		void setReached(bool reached) { m_Reached = reached; };
		void setTarget(cocos2d::Vec2 target) { m_Target = target; };
		void setLeader(GameObject* leader) { m_Leader = leader; };
		void setFollow(bool follow) { m_Follow = follow; if (follow) { m_Leading = false; } };
		void setLeading(cocos2d::Vec2 target, int form, float rest) {m_Target = target;m_TargetNode = target; m_Leading = true; m_Formation = form; m_Rest = rest; };
		void setLeading(bool leading) { m_Leading = leading; };
		void setGameObjects(std::map<int, GameObject*> objects) { m_GameObjects = objects; };
		void setSelected(bool selected) { m_Selected = selected; };
		void setSpeed(float speed) { m_Speed = speed; };
		void setFlockingWeights(float alignment, float cohesion, float seperation, float steerToLeader, float currentVector) { m_AWeight = alignment; m_CWeight = cohesion; m_SWeight = seperation; m_LWeight = steerToLeader; m_VWeight = currentVector; };
		void stopFollow();
		void setFormation(int formation) { m_Formation = formation; };
		void setRestAngle(float rest) { m_Rest = rest; };
		void clearNodes();
		void setFree(bool free) { m_Free = free; };
		void setAngle(float angle) { m_Body->SetTransform(m_Body->GetPosition(), angle * DTR_RATIO); };
		void setHasTarget(bool hasTarget) { m_HasTarget = hasTarget; };
		void setHalted(bool halted) { m_Halted = halted; };

		b2Body* getBody() { return m_Body; };

		GameObject* getLeader() { return m_Leader; };

		bool getSelected() { return m_Selected; };
		bool getReached() { return m_Reached; };
		bool getLeading() { return m_Leading; }
		bool getFollow() { return m_Follow; };
		bool getFinished() { return m_Finished; };
		bool getRested() { return m_Rested; };
		bool getFree() { return m_Free; };
		bool getHasTarget() { return m_HasTarget; };
		bool getHalted() { return m_Halted; };
		
		MyBullet* m_Bullet;
		bool m_ButtonActive;
		bool m_Engauged;

	private:

		// PRIVATE FUNCTIONS

		void rotate();

		float distance(cocos2d::Vec2 position, cocos2d::Vec2 target);
		float setSpeed();
		float getAngleDiff(float x, float y);
		float normalise(float angle);
		float getSpace() { return m_Space; };

		cocos2d::Vec2 alignment();
		cocos2d::Vec2 cohesion();
		cocos2d::Vec2 seperation();
		cocos2d::Vec2 leader();
		cocos2d::Vec2 vector();
		cocos2d::Vec2 unitVector(cocos2d::Vec2 vector);
		cocos2d::Vec2 neighbour(std::map<int, GameObject*>& objects);


		// VARIABLES AND MEMBER OBJECTS

		cocos2d::Size m_Size;

		b2FixtureDef m_BodyFixture;
		b2BodyDef m_BodyDef;
		b2CircleShape m_Circle;
		b2PolygonShape m_Polygon;
		b2Body* m_Body;

		enum m_Category {OBJECT = 0x0001};

		SessionData* m_SessionData;
		Utils* m_Utils;

        std::string m_File;

		cocos2d::Vec2 m_Target;
		cocos2d::Vec2 m_TargetNode;
		cocos2d::Vec2 m_Position;

		float m_Pixel;
		float m_Angle;
		float m_Rest;
		float m_ToAngle;
		float m_Steer;
		float m_Speed;
		float m_MaxSpeed;
		float m_Distance;
		float m_Space;
		

		float m_AWeight;
		float m_CWeight;
		float m_SWeight;;
		float m_LWeight;
		float m_VWeight;

		bool m_Reached;
		bool m_Rested;
		bool m_Selected;
		bool m_ClockWise;
		bool m_Follow;
		bool m_Flocking;
		bool m_Leading;
		bool m_Cohesion;
		bool m_LinearLeader;
		bool m_Finished;
		bool m_Free;
		bool m_HasTarget;
		bool m_Halted;

		int m_Formation;
		int m_ID;

		GameObject* m_Leader;

		std::vector<cocos2d::Node*> m_Nodes;
		std::vector<cocos2d::Node*>::iterator m_Nodes_Iter;

		std::map<int, GameObject*> m_GameObjects;
		std::map<int, GameObject*>::iterator m_GameObjects_Iter;

};
#endif // GADDBOX_GAMEOBJECT_H