#include "Marker.h"

// THIS FUCNTION TAKES THE SELECTED COUNT AND CREATES A SQUARE SHAPE - IT REPRESENTS A PLACEHOLDER FOR THE OBJECTS THAT ARE TO BE MOVED
bool Marker::init(int count, int type) {

	m_Type = type;

	int x;
	int y;

	m_Space = 25.0f;

	m_Count = 0;

	this->setPosition(cocos2d::Vec2(0.0f, 0.0f));

	int row = round(std::sqrt(float(count)));

	int lastRow = count % row;

	float offset = float(row - 1) * (m_Space / 2);

	for (x = 0; x < row && m_Count < count - lastRow; x++) {

		for (y = 0; y < row && m_Count < count - lastRow; y++) {

			cocos2d::Node* node = cocos2d::Node::create();
			node->setPosition(cocos2d::Vec2(-(x * m_Space) + offset, (y * m_Space) - offset));
			node->setTag(0);
			m_Nodes.push_back(node);
			this->addChild(node, 1);

			m_Count++;

		}

	}

	if (m_Count < count) {

		float back = ((float(row) - float(lastRow)) * m_Space) / 2;

		for (y = 0; y < lastRow; y++) {

			cocos2d::Node* node = cocos2d::Node::create();
			node->setPosition(cocos2d::Vec2(-(float(x) * m_Space) + offset, ((float(y) * m_Space) + back) - offset));
			node->setTag(0);
			m_Nodes.push_back(node);
			this->addChild(node, 1);

			m_Count++;

		}

	}

	return true;

}

// DRAW ALL NODES IN VECTOR
void Marker::update(float distance) {


	for (auto node : m_Nodes) {

		m_SessionData->getInstance()->getDrawNode()->drawCircle(node->getParent()->convertToWorldSpace(node->getPosition()), 5, 0, 12, false, cocos2d::Color4F(1.0f, 0.0f, 0.0f, 1.0f));

	}

}

// CLEAR ALL NODES IN VECTOR
void Marker::clear() {

	for (m_Nodes_Iter = m_Nodes.begin(); m_Nodes_Iter != m_Nodes.end();) {
	
		delete (*m_Nodes_Iter);
		m_Nodes_Iter = m_Nodes.erase(m_Nodes_Iter);
	
	}

}

// GET DISTANCE BETWEEN TWO POINTS
float Marker::distance() {

	float x_diff = this->getPosition().x - m_Target.x;
	float y_diff = this->getPosition().y - m_Target.y;

	float diff = std::sqrt(x_diff * x_diff + y_diff * y_diff);

	return diff;

}

// GET ANGLE DIFFERENCE
float Marker::getAngleDiff(float x, float y) {

	if (x == y) {

		return 0.0f;

	}

	else if (x == 0.0f) {

		if (y < 0.0f) {

			m_ClockWise = false;
			return -(y);

		}

		else {

			m_ClockWise = true;
			return y;

		}

	}

	else if (x < 0.0f) {

		if (y == 0.0f) {

			m_ClockWise = true;
			return -x;

		}

		else if (y < 0.0f) {

			if (x < y) {

				m_ClockWise = true;
				return -x - (-y);

			}
			else {

				m_ClockWise = false;
				return -y - (-x);

			}

		}

		else if (y > 0.0f) {

			if (-x + y < 180) {

				m_ClockWise = true;
				return -x + y;

			}
			else {

				m_ClockWise = false;
				return (180 - (-x)) + (180 - y);

			}

		}

	}

	else if (x > 0.0f) {

		if (y == 0.0f) {

			m_ClockWise = false;
			return x;

		}

		else if (y > 0.0f) {

			if (x > y) {

				m_ClockWise = false;
				return x - y;

			}
			else {

				m_ClockWise = true;
				return y - x;

			}

		}

		else if (y < 0.0f) {


			if (x + (-y) < 180) {

				m_ClockWise = false;
				return x + (-y);

			}
			else {

				m_ClockWise = true;
				return (180 - (-y)) + (180 - x);

			}

		}

	}

	return 0.0f;

}

// ROTATE MAIN NODE
void Marker::rotate() {

	float difference = getAngleDiff(this->getRotation(), m_Rest);

	if (difference > 0.1f) {

		if (difference < 0.5) {

			if (m_ClockWise) { this->setRotation(m_Rest); }
			else { this->setRotation(m_Rest); }

		}
		else if (difference < 1.0) {

			if (m_ClockWise) { this->setRotation(this->getRotation() + 0.01f); }
			else { this->setRotation(this->getRotation() - 0.01f); }

		}
		else {

			if (m_ClockWise) { this->setRotation(this->getRotation() + m_Steer); }
			else { this->setRotation(this->getRotation() - m_Steer); }

		}

	}

	else if (difference <= 0.1f) {

		m_Finished = true;

	}

}
